------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Archer             --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local assets    = chunks.assets
local pool      = chunks.pool
local enemy     = chunks.enemy
local particles = chunks.particles
local actions   = chunks.actions
local effects   = chunks.effects
local utility   = chunks.utility
local textures  = chunks.textures
local sound     = chunks.sound

-- LuaArcher --
LuaArcher = chunks.class.inherit( chunks.enemy.LuaEnemy )
{
	offset       = -131.5,
	arrow_offset = v3f( 10.0, 15.0, 0.0 )
}

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaArcher )

-- The created method.
function LuaArcher:created( owner )
	-- Initialize super.
	enemy.LuaEnemy.created( self, owner )
	
	-- Initialize owner.
	owner.label    = "archer"
	owner.location = pool.get_v3f( self.offset, 0.0, 0.0 )
	
	-- Thrower.
	do
		local actor = owner:spawn( CollidableActor() )
		
		textures.setTexture( actor, assets.textures.archer )

		self.visuals.archer = actor
	end
	
	self:setVisual( "archer" )
	
	-- We're not killable.
	self.killable = false
end

-- The collided event.
function LuaArcher:onCollided( other )
	-- Spawn particles.
	do
		particles.spawn_normal( self.owner:getGame().lua.root, 
		                        assets.textures.particles_spark, 
		                        self.visual:getWorldLocation() )
	end
	
	-- Spawn slice.
	do
		effects.spawn_flash( self.owner:getGame().lua.root, 
		                     assets.textures.effects_slice, 
		                     self.visual:getWorldLocation() )
	end
	
	self.owner:clearActions()
	actions.shrink_and_kill( self.visual )
	
	-- Play sound.
	sound.play_random( assets.sounds.hit_enemy )
end

-- Setup easy thrower.
function LuaArcher:easy()
	local actor = self.owner
	
	-- Actions
	do
		local s = SequenceAction()
		local f = function( inner )
			local projectile = actor:spawn( chunks.arrow.spawn() )
			
			projectile.location = self.arrow_offset
			
			-- Actions
			do
				local s2 = SequenceAction()
				
				s2:addAction( MoveOffsetAction( utility.frandom( 2.0, 2.5 ), 
							  InterpAction.Lerp, pool.get_v3f( 350.0, 0.0, 0.0 ) ) )
				
				projectile:addAction( s2 )
			end
			
			-- Play sound.
			sound.play_random( assets.sounds.shoot_arrow )
		end
		
		s:addAction( SleepAction( 0.25 ) )
		s:addAction( LuaAction( 0.0, f ) )
		
		actor:addAction( s )
	end
end

-- Setup hard thrower.
function LuaArcher:hard()
	local actor = self.owner
	
	-- Actions
	do
		local s = SequenceAction()
		local f = function( inner )
			local delay = utility.frandom( 1.4, 1.8 )
			local g     = function( offset )
				local projectile = actor:spawn( chunks.arrow.spawn() )
			
				projectile.location = self.arrow_offset
				
				-- Actions
				do
					local split = SplitAction()
					local s1    = SequenceAction()
					local s2    = SequenceAction()
					
					
					s1:addAction( MoveOffsetAction( delay, 
								  InterpAction.Lerp, pool.get_v3f( 350.0, 0.0, 0.0 ) ) )
					s2:addAction( SleepAction( 0.3 ) )
					s2:addAction( MoveOffsetAction( 0.25, 
					              InterpAction.Smooth, pool.get_v3f( 0.0, offset, 0.0 ) ) )
					split:addAction( s1 )
					split:addAction( s2 )
					
					projectile:addAction( split )
				end
			end
		
			local offset = utility.frandom( 25.0, 35.0 )
			
			g( offset )
			g( offset * -1 )
		
			-- Play sound.
			sound.play_random( assets.sounds.shoot_arrow )
		end
		
		s:addAction( SleepAction( 0.45 ) )
		s:addAction( LuaAction( 0.0, f ) )
		
		actor:addAction( s )
	end
end


