------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Arrow              --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local assets    = chunks.assets
local pool      = chunks.pool
local enemy     = chunks.enemy
local particles = chunks.particles
local actions   = chunks.actions
local textures  = chunks.textures
local sound     = chunks.sound
local utility   = chunks.utility

-- LuaArrow --
LuaArrow = chunks.class.inherit( chunks.enemy.LuaEnemy )()

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaArrow )

-- The created method.
function LuaArrow:created( owner )
	-- Initialize super.
	enemy.LuaEnemy.created( self, owner )
	
	-- Initialize owner.
	owner.label = "arrow"
	
	-- Thrower.
	do
		local actor = owner:spawn( CollidableActor() )
		
		textures.setTexture( actor, assets.textures.arrow )

		self.visuals.arrow = actor
	end
	
	self:setVisual( "arrow" )
end

-- The collided event.
function LuaArrow:onCollided( other, is_bonus )
	local game    = self.owner:getGame()
	local ninja   = game.lua.ninja
	local spawner = is_bonus and particles.spawn_uber or particles.spawn_normal
	
	-- Spawn feather particles.
	do
		spawner( self.owner:getGame().lua.root, 
				 assets.textures.particles_feather, 
				 self.visual:getWorldLocation() )
	end
	
	-- Spawn splinter particles.
	do
		spawner( self.owner:getGame().lua.root, 
				 assets.textures.particles_splinter, 
				 self.visual:getWorldLocation() )
	end
	
	self.owner:clearActions()
	actions.shrink_and_kill( self.visual )
	
	-- Play sound.
	sound.play_random( assets.sounds.hit_arrow )
end

-- The bonus setup method.
function LuaArrow:bonus( ninja )
	local owner  = self.owner
	local game   = owner:getGame()
	local offset = ninja.lua.sides.left.offset
	local dir    = ( ninja:getWorldLocation().x < 0.0 ) and 1 or -1
	local xdist  = 60.0
	local ydist  = 70.0
	local jumps  = 4
	local jdist  = 0.5
	local yend   = ( jumps + 2 ) * jdist * ydist + ninja.location.y - offset.y
	local delay  = 3.7
	local fdelay = 1.0
	
	-- Set the texture.
	textures.setTexture( ninja.lua.visual, assets.textures.bonus_archer )
	
	-- Enlarge shield.
	ninja.lua.shield.lua:bonusStart( 2.1 )
	
	-- Actions.
	do
		local s       = SequenceAction()	
		local d       = dir
		local pcent   = 1.0 / ( jumps + 1 )
		local set_rot = function( inner, d )
			local r = v3f( 0.0, ( d == -1 ) and 180.0 or 0.0, 0.0 )
			
			ninja.lua:setRotation( r, v3f( 0.0, r.y - 180.0, 0.0 ) )
		end
		local shoot = function( inner, d )
			local actor = game:getScene():spawn( SpriteActor() )
			
			-- Set data.
			textures.setTexture( actor, assets.textures.arrow )
			
			actor.location = ninja:getWorldLocation() + pool.get_v3f( 0.0, 25.0, -1 )
			actor.rotation = pool.get_v3f( 0.0, ( d == 1 ) and 180.0 or 0.0, -30.0 )

			-- Actions.
			do
				local s = SequenceAction()
				
				s:addAction( MoveOffsetAction( 1.5, InterpAction.Smooth,
				             pool.get_v3f( 250.0 * d * -1, -250.0, 0.0 ) ) )
				s:addAction( ScaleAction( 0.1, InterpAction.Smooth,
				             pool.get_v3f( 0.0, 0.0, 1.0 ) ) )
				s:addAction( DestroyAction( 0.0 ) )
				
				actor:addAction( s )
			end
		end	
		local jump = function( hdist, ydistup, ydistdown, delay )
			local split = SplitAction()
			local s1    = SequenceAction()
			local s2    = SequenceAction()
			
			-- Horizontal movement.
			s1:addAction( MoveOffsetAction( delay, InterpAction.Smooth,
			              pool.get_v3f( hdist, 0.0, 0.0 ) ) )
			              
			-- Vertical movement.
			s2:addAction( MoveOffsetAction( delay * 0.5, InterpAction.Smooth,
			              pool.get_v3f( 0.0, ydistup, 0.0 ) ) )
			s2:addAction( MoveOffsetAction( delay * 0.5, InterpAction.Smooth,
			              pool.get_v3f( 0.0, ydistdown, 0.0 ) ) )
		
			s:addAction( LuaAction( 0.0, set_rot, d ) )
			s:addAction( LuaAction( 0.0, shoot, d ) )
			split:addAction( s1 )
			split:addAction( s2 )
			s:addAction( split )
		end
		
		-- Initial jump.
		jump( ( math.abs( ninja.location.x ) + xdist ) * d, ydist, ydist * -jdist, delay * pcent )
		
		d = d * -1
		
		-- Middle jumps.
		do
			for i = 1,jumps do
				jump( xdist * 2.0 * d, ydist, ydist * -jdist, delay * pcent )
				
				d = d * -1
			end
		end
		
		-- Final jump.
		jump( ( math.abs( offset.x ) + xdist ) * d, ydist, -yend, fdelay )
		
		-- Done.
		s:addAction( LuaMemberAction( 0.0, "bonusStop" ) )	
		
		ninja:addAction( s )
	end
	
	-- Play sound.
	sound.play_random( assets.sounds.bonus_archer )
	
	return delay + fdelay
end

-- Update the ninja's trailer.
function LuaArrow:updateTrailer( trailer )
	local s = SequenceAction()
			
	s:addAction( ColorAction( 0.4, InterpAction.Smooth, 
	             pool.get_colorf( 255.0 / 255.0, 255.0 / 255.0, 255.0 / 255.0, 1.0 ) ) )
	
	trailer:addAction( s )	
end
