------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Bee                --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local assets    = chunks.assets
local pool      = chunks.pool
local enemy     = chunks.enemy
local particles = chunks.particles
local actions   = chunks.actions
local textures  = chunks.textures
local sound     = chunks.sound
local utility   = chunks.utility

-- LuaBee --
LuaBee = chunks.class.inherit( chunks.enemy.LuaEnemy )()

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaBee )

-- The created method.
function LuaBee:created( owner )
	-- Initialize super.
	enemy.LuaEnemy.created( self, owner )
	
	-- Initialize owner.
	owner.label = "bee"
	
	-- Bee.
	do
		local actor = owner:spawn( CollidableActor() )
		
		textures.setTexture( actor, assets.textures.bee )

		self.visuals.bee = actor
	end
	
	self:setVisual( "bee" )
end

-- The collided event.
function LuaBee:onCollided( other, is_bonus )
	local game  = self.owner:getGame()
	local ninja = game.lua.ninja
	
	self.owner:clearActions()
	actions.shrink_and_kill( self.owner )
	
	-- Play sound.
	sound.play_random( assets.sounds.hit_bee )
end

-- Give the bee a random trajectory, then kill it.
function LuaBee:randomize()
	local owner = self.owner
	local range = function() return utility.frandom( 30.0, 40.0 ) end
	local delay = function() return utility.frandom( 0.15, 0.25 ) end
	local s     = SequenceAction()
	local s1    = SequenceAction()
	local s2    = SequenceAction()
	local split = SplitAction()
	
	for i = 1,4 do
		local next = utility.random_normal( 0.0, 360.0 ) * range()
		local perp = v2f( next.y, next.x ) * utility.frandom( 0.4, 0.6 )
		local time = delay()
		
		-- Invert an axis on perp.
		if math.random() < 0.5 then
			perp.x = -1.0 * perp.x
		else
			perp.y = -1.0 * perp.y
		end
		
		-- Actions.
		do
			s1:addAction( MoveOffsetAction( time, InterpAction.Lerp,
			                                pool.get_v3f( next.x, next.y, 0.0 ) ) )

			s2:addAction( MoveOffsetAction( time / 2.0, InterpAction.Smooth,
			                                pool.get_v3f( perp.x, perp.y, 0.0 ) ) )
			s2:addAction( MoveOffsetAction( time / 2.0, InterpAction.Smooth,
			                                pool.get_v3f( -1.0 * perp.x, -1.0 * perp.y, 0.0 ) ) )
		end		
	end
	
	-- Add the root action to the bee.
	split:addAction( s1 )
	split:addAction( s2 )
	s:addAction( split )
	s:addAction( ScaleAction( 0.1, InterpAction.Smooth, pool.get_v3f( 0.0, 0.0, 0.0 ) ) )
	s:addAction( DestroyAction( 0.0 ) )
	owner:addAction( s )
end

-- The bonus setup method.
function LuaBee:bonus( ninja )
	local game          = ninja:getGame()
	local offset        = ninja.lua.sides.left.offset
	local ldist         = math.abs( ninja.location.x - offset.x )
	local rdist         = math.abs( ninja.location.x + offset.x )
	local dir           = ldist < rdist and -1 or 1
	local initial_delay = 0.2
	local delay         = 0.9
	local hoffset       = 120.0
	
	textures.setTexture( ninja.lua.visual, assets.textures.bonus_bee )
	
	-- Enlarge shield.
	ninja.lua.shield.lua:bonusStart( 2.5 )
	
	-- Actions.
	do
		local s     = SequenceAction()
		local split = SplitAction()
		local s1    = SequenceAction()
		local s2    = SequenceAction()
		
		s:addAction( MoveAction( initial_delay, InterpAction.Smooth,
		                         pool.get_v3f( math.abs( offset.x ) * dir, offset.y, offset.z ) ) )
		
		for i = 1,3 do
			dir = dir * -1
			
			s1:addAction( MoveOffsetAction( delay, InterpAction.Smooth,
			                                pool.get_v3f( math.abs( offset.x ) * 2.0 * dir, 0.0, 0.0 ) ) )
			s2:addAction( MoveOffsetAction( delay / 4.0, InterpAction.Smooth,
										    pool.get_v3f( 0.0, 70.0, 0.0 ) ) )
			s2:addAction( MoveOffsetAction( delay / 4.0, InterpAction.Smooth,
										    pool.get_v3f( 0.0, -70.0, 0.0 ) ) )                  
		    s2:addAction( MoveOffsetAction( delay / 4.0, InterpAction.Smooth,
										    pool.get_v3f( 0.0, 70.0, 0.0 ) ) )
			s2:addAction( MoveOffsetAction( delay / 4.0, InterpAction.Smooth,
										    pool.get_v3f( 0.0, -70.0, 0.0 ) ) )
		end
		
		split:addAction( s1 )		
		split:addAction( s2 )
		s:addAction( split )
		s:addAction( LuaMemberAction( 0.0, "bonusStop" ) )	
		
		ninja:addAction( s )
	end
	
	-- Play sounds.
	sound.play( assets.sounds.bonus_trigger_bee )
	sound.play( assets.sounds.bonus_loop_bee )
	
	return initial_delay + 3.0 * delay 
end

-- Update the ninja's trailer.
function LuaBee:updateTrailer( trailer )
	local s = SequenceAction()
			
	s:addAction( ColorAction( 0.4, InterpAction.Smooth, pool.get_colorf( 255.0 / 255.0, 205.0 / 255.0, 0.0 / 255.0, 1.0 ) ) )
			
	trailer:addAction( s )	
end
