------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Beehive            --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local assets    = chunks.assets
local pool      = chunks.pool
local enemy     = chunks.enemy
local particles = chunks.particles
local actions   = chunks.actions
local textures  = chunks.textures
local sound     = chunks.sound
local utility   = chunks.utility
local effects   = chunks.effects

-- LuaBeehive --
LuaBeehive = chunks.class.inherit( chunks.enemy.LuaEnemy )
{
	exploded = false
}

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaBeehive )

-- The created method.
function LuaBeehive:created( owner )
	-- Initialize super.
	enemy.LuaEnemy.created( self, owner )
	
	-- Initialize owner.
	owner.label = "beehive"
	
	-- Thrower.
	do
		local actor = owner:spawn( CollidableActor() )
		
		textures.setTexture( actor, assets.textures.beehive )

		self.visuals.beehive = actor
	end
	
	self:setVisual( "beehive" )
end

-- The collided event.
function LuaBeehive:onCollided( other, is_bonus )
	-- If we've already exploded, do nothing.
	if self.exploded then
		return
	else
		self.exploded = true
	end
	
	local game    = self.owner:getGame()
	local ninja   = game.lua.ninja
	local spawner = is_bonus and particles.spawn_uber or particles.spawn_normal
	
	-- Spawn particles.
	do
		spawner( self.owner:getGame().lua.root, 
				 assets.textures.particles_honey, 
				 self.visual:getWorldLocation() )
	end
	
	-- Spawn honey.
	do
		local flash = game.lua.root:spawn( SpriteActor() )
		
		flash.location = self.visual:getWorldLocation()
		flash.color    = pool.get_colorf( 1.0, 1.0, 1.0, 0.0 )
		flash.scale    = pool.get_v3f( 0.5, 0.5, 1.0 )
		
		textures.setTexture( flash, assets.textures.effects_honey )
		
		-- Actions.
		do
			local s     = SequenceAction()
			local s2    = SequenceAction()
			local s3    = SequenceAction()
			local split = SplitAction()
			local delay = is_bonus and 0.8 or 0.5
			
			s2:addAction( ScaleAction( delay / 2.0, InterpAction.Smooth, pool.get_v3f( 1.5, 1.5, 1.0 ) ) )
			s2:addAction( ScaleAction( delay / 2.0, InterpAction.Smooth, pool.get_v3f( 0.5, 0.5, 1.0 ) ) )
			split:addAction( s2 )
			
			s3:addAction( AlphaAction( delay * 0.2, InterpAction.Smooth, 1.0 ) )
			s3:addAction( SleepAction( delay * 0.6 ) )
			s3:addAction( AlphaAction( delay * 0.2, InterpAction.Smooth, 0.0 ) )
			split:addAction( s3 )
			
			s:addAction( split )
			s:addAction( DestroyAction( 0.0 ) )
			
			flash:addAction( s )
		end
	end
	
	self.owner:clearActions()
	actions.shrink_and_kill( self.visual )
	
	-- Play sound.
	sound.play_random( assets.sounds.hit_beehive )
end

-- The explode event.
function LuaBeehive:explode()
	-- If we've already exploded, do nothing.
	if self.exploded then
		return
	else
		self.exploded = true
	end
	
	local root  = self.owner:getGame().lua.root
	local actor = self.visual
	local world = actor:getWorldLocation()
	
	-- Spawn bees.
	do
		for i = 1,4 do
			local bee = root:spawn( chunks.bee.spawn() )
			
			bee.location = world
			bee.lua:randomize()
		end
	end
	
	-- Need to unregister or ninja collision will fail.
	self:unregister()
	
	-- Shrink and kill.
	actions.shrink_and_kill( actor )

	-- Play sound.
	sound.play_random( assets.sounds.beehive_explode )
end

-- The bonus setup method.
function LuaBeehive:bonus( ninja )
	local game          = ninja:getGame()
	local offset        = ninja.lua.sides.left.offset
	local ldist         = math.abs( ninja.location.x - offset.x )
	local rdist         = math.abs( ninja.location.x + offset.x )
	local dir           = ldist < rdist and -1 or 1
	local initial_delay = 0.2
	local delay         = 0.9
	local hoffset       = 120.0
	
	textures.setTexture( ninja.lua.visual, assets.textures.bonus_bee )
	
	-- Enlarge shield.
	ninja.lua.shield.lua:bonusStart( 2.5 )
	
	-- Actions.
	do
		local s     = SequenceAction()
		local split = SplitAction()
		local s1    = SequenceAction()
		local s2    = SequenceAction()
		
		s:addAction( MoveAction( initial_delay, InterpAction.Smooth,
		                         pool.get_v3f( math.abs( offset.x ) * dir, offset.y, offset.z ) ) )
		
		for i = 1,7 do
			dir = dir * -1
			
			s1:addAction( MoveOffsetAction( delay, InterpAction.Smooth,
			                                pool.get_v3f( math.abs( offset.x ) * 2.0 * dir, 0.0, 0.0 ) ) )
		end
		
		s2:addAction( MoveOffsetAction( delay * 3.5, InterpAction.Smooth,
		                                pool.get_v3f( 0.0, 300.0, 0.0 ) ) )
		s2:addAction( MoveOffsetAction( delay * 3.5, InterpAction.Smooth,
		                                pool.get_v3f( 0.0, -300.0, 0.0 ) ) )                        
		split:addAction( s1 )		
		split:addAction( s2 )
		s:addAction( split )
		s:addAction( LuaMemberAction( 0.0, "bonusStop" ) )	
		
		ninja:addAction( s )
	end
	
	-- Spawn bee swarm.
	do
		local system = particles.spawn_normal( ninja, 
		                                       assets.textures.bee, 
		                                       pool.get_v3f( 0.0, 0.0, -1.0 ) )
		
		system.lifetime               = initial_delay + 7.0 * delay 
		system.initial_rotation_range = pool.get_v2f( 0.0, 0.0 )
		system.rotation_speed_range   = pool.get_v2f( 0.0, 0.0 )
		system.follow_system          = true
	end
	
	-- Play sounds.
	sound.play( assets.sounds.bonus_trigger_beehive )
	sound.play( assets.sounds.bonus_loop_beehive )
	
	return initial_delay + 7.0 * delay 
end

-- Update the ninja's trailer.
function LuaBeehive:updateTrailer( trailer )
	local s = SequenceAction()
			
	s:addAction( ColorAction( 0.4, InterpAction.Smooth, pool.get_colorf( 255.0 / 255.0, 205.0 / 255.0, 0.0 / 255.0, 1.0 ) ) )
	
	trailer:addAction( s )	
end
