------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Beetle             --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local assets    = chunks.assets
local pool      = chunks.pool
local enemy     = chunks.enemy
local particles = chunks.particles
local actions   = chunks.actions
local utility   = chunks.utility
local textures  = chunks.textures
local sound     = chunks.sound

-- LuaBeetle --
LuaBeetle = chunks.class.inherit( chunks.enemy.LuaEnemy )()

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaBeetle )

-- The created method.
function LuaBeetle:created( owner )
	-- Initialize super.
	enemy.LuaEnemy.created( self, owner )
	
	-- Initialize owner.
	owner.label = "beetle"
	
	-- Thrower.
	do
		local actor = owner:spawn( CollidableActor() )
		
		textures.setTexture( actor, assets.textures.beetle )

		self.visuals.beetle = actor
	end
	
	self:setVisual( "beetle" )
end

-- Spawn wing explosion.
function LuaBeetle:spawnWingExplosion( game, spawner, location )
	-- Spawn particles (wing_1).
	do
		spawner( game.lua.root, 
				 assets.textures.particles_wing_1, 
				 location )
	end
	
	-- Spawn particles (wing_2).
	do
		spawner( game.lua.root, 
				 assets.textures.particles_wing_2, 
				 location )
	end
end

-- The collided event.
function LuaBeetle:onCollided( other, is_bonus )
	local game  = self.owner:getGame()
	local ninja = game.lua.ninja
	
	-- Wing explosion.
	self:spawnWingExplosion( game, particles.spawn_normal, self.visual:getWorldLocation() )
	
	self.owner:clearActions()
	actions.shrink_and_kill( self.visual )
	
	-- Play sound.
	sound.play_random( assets.sounds.hit_beetle )
end

-- Setup easy sequence.
function LuaBeetle:easy()
	local actor = self.owner
	local game  = actor:getGame()
	local ninja = game.lua.ninja
	local hdist = 200.0
	local vdist = 350.0
	local delay = 3.5
	
	actor.location = pool.get_v3f( -50.0, -280.0, 0.0 )
	
	-- Play sound.
	sound.play_random( assets.sounds.beetle_enter )
	
	-- Actions
	do
		local s     = SequenceAction()
		local split = SplitAction()
		local s2    = SequenceAction()
		
		split:addAction( MoveOffsetAction( delay, InterpAction.Lerp, pool.get_v3f( hdist, 0.0, 0.0 ) ) )
		s2:addAction( MoveOffsetAction( delay / 2.0, InterpAction.Smooth, pool.get_v3f( 0.0, vdist, 0.0 ) ) )
		s2:addAction( MoveOffsetAction( delay / 2.0, InterpAction.Smooth, pool.get_v3f( 0.0, -vdist, 0.0 ) ) )
		split:addAction( s2 )
		s:addAction( split )
		
		actor:addAction( s )
	end	
end

-- Setup hard sequence.
function LuaBeetle:hard()
	local actor = self.owner
	local game  = actor:getGame()
	local ninja = game.lua.ninja
	local hdist = 200.0
	local vdist = 350.0
	local delay = 2.0
	
	actor.location = pool.get_v3f( -50.0, -280.0, 0.0 )
	
	-- Play sound.
	sound.play_random( assets.sounds.beetle_enter )
	
	-- Actions
	do
		local s     = SequenceAction()
		local split = SplitAction()
		local s2    = SequenceAction()
		
		split:addAction( MoveOffsetAction( delay, InterpAction.Lerp, pool.get_v3f( hdist, 0.0, 0.0 ) ) )
		s2:addAction( MoveOffsetAction( delay / 2.0, InterpAction.Smooth, pool.get_v3f( 0.0, vdist, 0.0 ) ) )
		s2:addAction( MoveOffsetAction( delay / 2.0, InterpAction.Smooth, pool.get_v3f( 0.0, -vdist, 0.0 ) ) )
		split:addAction( s2 )
		s:addAction( split )
		
		actor:addAction( s )
	end	
end

-- The bonus setup method.
function LuaBeetle:bonus( ninja )
	local game        = ninja:getGame()
	local offset      = ninja.lua.sides.left.offset
	local target      = math.random() < 0.5 and offset or v3f( offset.x * -1, offset.y, offset.z )
	local delay       = 3.0
	local sleep_delay = 1.0
	local kill_delay  = 0.4
	
	textures.setTexture( ninja.lua.visual, assets.textures.bonus_beetle )
	
	-- Enlarge shield.
	ninja.lua.shield.lua:bonusStart( 2.7 )
	
	-- Actions.
	do
		local s = SequenceAction()
		local f = function( inner )
			-- Wing explosion.
			self:spawnWingExplosion( game, particles.spawn_uber, inner:getWorldLocation() )
		end
		
		ninja.location = pool.get_v3f( 0.0, -320.0, offset.z )
		s:addAction( MoveAction( delay, InterpAction.Smooth, 
		                         pool.get_v3f( 0.0, 0.0, offset.z ) ) )		
		--s:addAction( SleepAction( sleep_delay ) )
		s:addAction( ScaleAction( sleep_delay / 4.0, InterpAction.Smooth,
		                          pool.get_v3f( 0.8, 0.8, 1.0 ) ) )
		s:addAction( ScaleAction( sleep_delay / 4.0, InterpAction.Smooth,
		                          pool.get_v3f( 1.0, 1.0, 1.0 ) ) )
		s:addAction( ScaleAction( sleep_delay / 4.0, InterpAction.Smooth,
		                          pool.get_v3f( 0.6, 0.6, 1.0 ) ) )
		s:addAction( ScaleAction( sleep_delay / 4.0, InterpAction.Smooth,
		                          pool.get_v3f( 1.2, 1.2, 1.0 ) ) )
		s:addAction( LuaAction( 0.0, f ) )
		s:addAction( ScaleAction( kill_delay, InterpAction.Smooth, pool.get_v3f( 0.0, 0.0, 1.0 ) ) )
		s:addAction( MoveAction( 0.0, InterpAction.Lerp, target ) )
		s:addAction( ScaleAction( 0.0, InterpAction.Smooth, pool.get_v3f( 1.0, 1.0, 1.0 ) ) )
		s:addAction( LuaMemberAction( 0.0, "bonusStop" ) )	
		
		ninja:addAction( s )
	end
	
	-- Play sound.
	sound.play( assets.sounds.bonus_trigger_beetle )
	sound.play( assets.sounds.bonus_loop_beetle )
	
	return delay + sleep_delay + kill_delay
end

-- Update the ninja's trailer.
function LuaBeetle:updateTrailer( trailer )
	local s = SequenceAction()
			
	s:addAction( ColorAction( 0.4, InterpAction.Smooth, pool.get_colorf( 80.0 / 255.0, 229.0 / 255.0, 244.0 / 255.0, 1.0 ) ) )
			
	trailer:addAction( s )	
end