------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Bird               --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local assets    = chunks.assets
local pool      = chunks.pool
local enemy     = chunks.enemy
local particles = chunks.particles
local actions   = chunks.actions
local utility   = chunks.utility
local textures  = chunks.textures
local sound     = chunks.sound

-- LuaBird --
LuaBird = chunks.class.inherit( chunks.enemy.LuaEnemy )()

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaBird )

-- The created method.
function LuaBird:created( owner )
	-- Initialize super.
	enemy.LuaEnemy.created( self, owner )
	
	-- Initialize owner.
	owner.label = "bird"
	
	-- Thrower.
	do
		local actor = owner:spawn( CollidableActor() )
		
		textures.setTexture( actor, assets.textures.bird )

		self.visuals.bird = actor
	end
	
	self:setVisual( "bird" )
end

-- The collided event.
function LuaBird:onCollided( other, is_bonus )
	local game    = self.owner:getGame()
	local ninja   = game.lua.ninja
	local spawner = is_bonus and particles.spawn_uber or particles.spawn_normal
	
	-- Spawn particles.
	do
		spawner( game.lua.root, 
		         assets.textures.particles_feather, 
		         self.visual:getWorldLocation() )
	end
	
	actions.shrink_and_kill( self.visual )
	
	-- Play sound.
	sound.play_random( assets.sounds.hit_bird )
end

-- Setup easy sequence.
function LuaBird:easy()
	local actor = self.owner
	local game  = actor:getGame()
	local ninja = game.lua.ninja
	local delay = 0.1
	
	actor.location = pool.get_v3f( -132.0, 270.0, 0.0 )
	
	-- Actions
	do
		local roost        = actor.location + v3f( 0.0, utility.frandom( -90.0, -110.0 ), 0.0 )
		local right_offset = ninja.lua.sides.right.offset
		local offset       = v3f( right_offset.x, right_offset.y - 20.0, 0.0 )
        local a            = getOwnerToWorldMxTimesLocation( actor, roost )
        local b            = getOwnerToWorldMxTimesLocation( ninja, offset )
		local target       = ( b - a ):normalized() * 550.0
		local s            = SequenceAction()
		
		target.z = 0.0
		
		s:addAction( SleepAction( delay ) )
		s:addAction( MoveAction( utility.frandom( 0.4, 0.7 ), InterpAction.Smooth, roost ) )
		s:addAction( SleepAction( utility.frandom( 0.8, 1.4 ) ) )
		s:addAction( MoveAction( utility.frandom( 1.5, 1.7 ), InterpAction.Lerp, target ) )
		
		actor:addAction( s )
	end	
end

-- Setup hard sequence.
function LuaBird:hard()
	local actor = self.owner
	local game  = actor:getGame()
	local ninja = game.lua.ninja
	local delay = 0.1
	
	actor.location = pool.get_v3f( -132.0, 270.0, 0.0 )
	
	-- Actions
	do
		local roost        = actor.location + v3f( 0.0, utility.frandom( -90.0, -110.0 ), 0.0 )
		local right_offset = ninja.lua.sides.right.offset
		local offset       = v3f( right_offset.x, right_offset.y - 20.0, 0.0 )
        local a            = getOwnerToWorldMxTimesLocation( actor, roost )
        local b            = getOwnerToWorldMxTimesLocation( ninja, offset )
		local target       = ( b - a ):normalized() * 550.0
		local s            = SequenceAction()
		
		target.z = 0.0
		
		s:addAction( SleepAction( delay ) )
		s:addAction( MoveAction( utility.frandom( 0.4, 0.7 ), InterpAction.Smooth, roost ) )
		s:addAction( SleepAction( utility.frandom( 0.8, 1.4 ) ) )
		s:addAction( MoveAction( utility.frandom( 1.2, 1.4 ), InterpAction.Lerp, target ) )
		
		actor:addAction( s )
	end	
end

-- The bonus setup method.
function LuaBird:bonus( ninja )
	local left_offset = ninja.lua.sides.left.offset
	local offset      = v3f( -80.0, 0.0, ninja.location.z )
	local center      = v3f( 0.0, -90.0, ninja.location.z )
	local dir         = ( ninja:getWorldLocation().x < 0.0 ) and 1 or -1
	local delay       = 1.0
	local set_rot     = function( inner, d )
		local r = ( d == -1 ) and 180.0 or 0.0
		
		ninja.lua:setRotation( v3f( 0.0, r, 0.0 ), v3f( 0.0, r + 180.0, 0.0 ) )
	end
	
	-- Set visual.
	textures.setTexture( ninja.lua.visual, assets.textures.bonus_bird )
	
	-- Enlarge shield.
	ninja.lua.shield.lua:bonusStart( 2.3 )
	
	-- Actions.
	do
		local s = SequenceAction()
		
		s:addAction( LuaAction( 0.0, set_rot, dir ) )
		s:addAction( MoveAction( delay, InterpAction.Smooth, center ) )
		s:addAction( MoveAction( delay, InterpAction.Smooth, 
								 pool.get_v3f( math.abs( offset.x ) * dir, offset.y, offset.z ) ) )
		dir = dir * -1
		s:addAction( LuaAction( 0.0, set_rot, dir ) )
		s:addAction( MoveAction( delay, InterpAction.Smooth, 
								 pool.get_v3f( math.abs( offset.x ) * dir, offset.y, offset.z ) ) )
		dir = dir * -1
		s:addAction( LuaAction( 0.0, set_rot, dir ) )
		s:addAction( MoveAction( delay, InterpAction.Smooth, center ) )
		s:addAction( MoveAction( delay, InterpAction.Smooth, 
		                         pool.get_v3f( math.abs( left_offset.x ) * dir, left_offset.y, left_offset.z ) ) )
		s:addAction( LuaMemberAction( 0.0, "bonusStop" ) )	
		
		ninja:addAction( s )
	end
	
	-- Play sounds.
	sound.play( assets.sounds.bonus_trigger_bird )
	sound.play( assets.sounds.bonus_loop_bird )
	
	return delay * 5
end

-- Update the ninja's trailer.
function LuaBird:updateTrailer( trailer )
	local s = SequenceAction()
			
	s:addAction( ColorAction( 0.4, InterpAction.Smooth, pool.get_colorf( 0.0, 160.0 / 255.0, 1.0, 1.0 ) ) )
			
	trailer:addAction( s )	
end