------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Boulder            --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local assets    = chunks.assets
local pool      = chunks.pool
local enemy     = chunks.enemy
local particles = chunks.particles
local actions   = chunks.actions
local textures  = chunks.textures
local sound     = chunks.sound
local utility   = chunks.utility
local constants = engine.platform.constants

-- LuaBoulder --
LuaBoulder = chunks.class.inherit( chunks.enemy.LuaEnemy )()

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaBoulder )

-- The created method.
function LuaBoulder:created( owner )
	-- Initialize super.
	enemy.LuaEnemy.created( self, owner )
	
	-- Initialize owner.
	owner.label = "boulder"
	
	-- Thrower.
	do
		local actor = owner:spawn( CollidableActor() )
		
		textures.setTexture( actor, assets.textures.boulder )

		self.visuals.boulder = actor
	end
	
	self:setVisual( "boulder" )
end

-- The collided event.
function LuaBoulder:onCollided( other, is_bonus )
	local game    = self.owner:getGame()
	local ninja   = game.lua.ninja
	local spawner = is_bonus and particles.spawn_uber or particles.spawn_normal
	
	-- Spawn rock1 particles.
	do
		spawner( self.owner:getGame().lua.root, 
				 assets.textures.particles_rock1, 
				 self.visual:getWorldLocation() )
	end
	
	-- Spawn rock2 particles.
	do
		spawner( self.owner:getGame().lua.root, 
				 assets.textures.particles_rock2, 
				 self.visual:getWorldLocation() )
	end
	
	self.owner:clearActions()
	actions.shrink_and_kill( self.visual )
	
	-- Play sound.
	sound.play_random( assets.sounds.hit_boulder )
end

-- The bonus setup method.
function LuaBoulder:bonus( ninja )
	local game        = ninja:getGame()
	local offset      = ninja.lua.sides.left.offset
	local dir         = ( ninja:getWorldLocation().x < 0.0 ) and -1 or 1
	local delay       = 2.2
	local sleep_delay = 0.3
	local rotation    = v3f( 0.0, ( dir == 1 ) and 180.0 or 0.0, 0.0 )
	
	-- Set visual.
	textures.setTexture( ninja.lua.visual, assets.textures.bonus_boulder )
	
	-- Enlarge shield.
	ninja.lua.shield.lua:bonusStart( 2.0 )
	
	-- Set the ninja's initial location.
	ninja.location = pool.get_v3f( ( math.abs( offset.x ) - 10.0 ) * dir, -310.0, offset.z ) 
	if constants.WIDESCREEN then
		ninja.location = ninja.location + v3f(0, -44, 0)
	end
		
	-- Set initial rotation.
	ninja.lua:setRotation( rotation, v3f( 0.0, rotation.y + 180.0, 0.0 ) )
	
	-- The cannon.
	do
		local fulcrum = game:getScene():spawn( ActorBase() )
		local actor   = fulcrum:spawn( SpriteActor() )
		
		textures.setTexture( actor, assets.textures.bonus_catapult )
		
		fulcrum.location = pool.get_v3f( 50.0 * dir * -1, -320.0, offset.z + 2.0 )
		fulcrum.rotation = rotation
		actor.location   = pool.get_v3f( math.abs( assets.textures.bonus_catapult.size.x ) * -0.4, -10.0, 0.0 )
		if constants.WIDESCREEN then
			actor.location = actor.location + v3f(0, -44, 0)
		end
		
		-- Fulcrum actions.
		do
			local s = SequenceAction()
			local d = delay - sleep_delay
			
			s:addAction( MoveOffsetAction( sleep_delay, InterpAction.Smooth,
			                               pool.get_v3f( 0.0, 90.0, 0.0 ) ) )
			s:addAction( RotateOffsetAction( d * 0.25, InterpAction.Smooth,
			             pool.get_v3f( 0.0, 0.0, -30.0 ) ) )
			s:addAction( SleepAction( d * 0.75 ) )
			s:addAction( MoveOffsetAction( sleep_delay, InterpAction.Smooth,
			                               pool.get_v3f( 0.0, -130.0, 0.0 ) ) )
			s:addAction( DestroyAction( 0.0 ) )                               
			
			fulcrum:addAction( s )
		end	
	end
	
	dir = dir * -1
	
	-- Actions.
	do
		local s     = SequenceAction()
		local split = SplitAction()
		local s1    = SequenceAction()
		local s2    = SequenceAction()

		s:addAction( MoveOffsetAction( sleep_delay, InterpAction.Smooth,
		             pool.get_v3f( 0.0, 90.0, 0.0 ) ) )
		
		s1:addAction( MoveAction( delay, InterpAction.Smooth, 
		                          pool.get_v3f( ( math.abs( offset.x ) ) * dir, offset.y, offset.z ) ) )
		s2:addAction( MoveOffsetAction( delay * 0.6, InterpAction.Smooth,
		                                pool.get_v3f( 0.0, 350.0, 0.0 ) ) )
		s2:addAction( MoveOffsetAction( delay * 0.4, InterpAction.Smooth,
		                                pool.get_v3f( 0.0, -360.0, 0.0 ) ) )                                    
		                                
		split:addAction( s1 )
		split:addAction( s2 )
		--split:addAction( s3 )
		s:addAction( split )
		s:addAction( LuaMemberAction( 0.0, "bonusStop" ) )
		
		ninja:addAction( s )
	end
	
	-- Play sounds.
	sound.play( assets.sounds.bonus_catapult )
	
	return sleep_delay + delay
end

-- Update the ninja's trailer.
function LuaBoulder:updateTrailer( trailer )
	local s = SequenceAction()
			
	s:addAction( ColorAction( 0.4, InterpAction.Smooth, 
	             pool.get_colorf( 255.0 / 255.0, 255.0 / 255.0, 0.0 / 255.0, 1.0 ) ) )
	
	trailer:addAction( s )	
end
