------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Cannon Ball        --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local assets    = chunks.assets
local pool      = chunks.pool
local enemy     = chunks.enemy
local particles = chunks.particles
local actions   = chunks.actions
local textures  = chunks.textures
local sound     = chunks.sound
local utility   = chunks.utility
local constants = engine.platform.constants

-- LuaCannonBall --
LuaCannonBall = chunks.class.inherit( chunks.enemy.LuaEnemy )()

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaCannonBall )

-- The created method.
function LuaCannonBall:created( owner )
	-- Initialize super.
	enemy.LuaEnemy.created( self, owner )
	
	-- Initialize owner.
	owner.label = "cannonball"
	
	-- Thrower.
	do
		local actor = owner:spawn( CollidableActor() )
		
		textures.setTexture( actor, assets.textures.cannonball )

		self.visuals.cannonball = actor
	end
	
	self:setVisual( "cannonball" )
end

-- The collided event.
function LuaCannonBall:onCollided( other, is_bonus )
	local game    = self.owner:getGame()
	local ninja   = game.lua.ninja
	local spawner = is_bonus and particles.spawn_uber or particles.spawn_normal
	
	-- Spawn particles.
	do
		local system = spawner( self.owner:getGame().lua.root, 
		                        assets.textures.particles_cannonball,
		                        self.visual:getWorldLocation() )

		do
			system.lifetime                  = utility.frandom( 0.05, 0.05 ) 
			system.particles_per_spawn_range = pool.get_v2i( 2, 2 )  
			system.spawn_frequency_range     = pool.get_v2f( 0.1, 0.1 )
		end
		
		system:clearScaleRanges()
	end
	
	self.owner:clearActions()
	actions.shrink_and_kill( self.visual )
	
	-- Play sound.
	sound.play_random( assets.sounds.hit_cannonball )
end

-- The bonus setup method.
function LuaCannonBall:bonus( ninja )
	local game        = ninja:getGame()
	local offset      = ninja.lua.sides.left.offset
	local dir         = ( ninja:getWorldLocation().x < 0.0 ) and -1 or 1
	local delay       = 2.2
	local sleep_delay = 0.3
	local rotation    = v3f( 0.0, ( dir == 1 ) and 180.0 or 0.0, 0.0 )
	
	-- Set visual.
	textures.setTexture( ninja.lua.visual, assets.textures.bonus_cannonball )
	
	-- Enlarge shield.
	ninja.lua.shield.lua:bonusStart( 2.4 )
	
	-- Set initial location.
	ninja.location = pool.get_v3f( ( math.abs( offset.x ) + 20.0 ) * dir, -320.0, offset.z ) 
	
	-- Set initial rotation.
	ninja.lua:setRotation( rotation, v3f( 0.0, rotation.y + 180.0, 0.0 ) )
	
	-- Set scale.
	ninja.scale = pool.get_v3f( 0.15, 0.15, 1.0 )
	
	-- The cannon.
	do
		local actor = game:getScene():spawn( SpriteActor() )
		
		textures.setTexture( actor, assets.textures.bonus_cannon )
		
		actor.location = pool.get_v3f( ( math.abs( offset.x ) ) * dir, -320.0, offset.z + 2.0 )
		if constants.WIDESCREEN then
			actor.location = actor.location + v3f(0, -44, 0)
		end
		
		actor.rotation = rotation
		
		-- Actions.
		do
			local s = SequenceAction()
			
			s:addAction( MoveOffsetAction( sleep_delay, InterpAction.Smooth,
			                               pool.get_v3f( 0.0, 120.0, 0.0 ) ) )
			s:addAction( SleepAction( delay - sleep_delay ) )
			s:addAction( MoveOffsetAction( sleep_delay, InterpAction.Smooth,
			                               pool.get_v3f( 0.0, -120.0, 0.0 ) ) )
			s:addAction( DestroyAction( 0.0 ) )                               
			
			actor:addAction( s )
		end	
	end
	
	dir = dir * -1
	
	-- Actions.
	do
		local s     = SequenceAction()
		local split = SplitAction()
		local s1    = SequenceAction()
		local s2    = SequenceAction()
		local s3    = SequenceAction()
		
		s:addAction( SleepAction( sleep_delay ) )
		s1:addAction( MoveAction( delay, InterpAction.Smooth, 
		                          pool.get_v3f( ( math.abs( offset.x ) + 10.0 ) * dir, offset.y, offset.z ) ) )
		s2:addAction( MoveOffsetAction( delay * 0.6, InterpAction.Smooth,
		                                pool.get_v3f( 0.0, 420.0, 0.0 ) ) )
		s2:addAction( MoveOffsetAction( delay * 0.4, InterpAction.Smooth,
		                                pool.get_v3f( 0.0, -370.0, 0.0 ) ) )                                    
		s3:addAction( ScaleAction( delay * 0.6, InterpAction.Smooth, 
		                           pool.get_v3f( 1.0, 1.0, 1.0 ) ) )
		split:addAction( s1 )
		split:addAction( s2 )
		split:addAction( s3 )
		s:addAction( split )
		s:addAction( LuaMemberAction( 0.0, "bonusStop" ) )
		
		ninja:addAction( s )
	end
	
	-- Play sounds.
	sound.play( assets.sounds.bonus_trigger_cannon )
	
	return sleep_delay + delay
end

-- Update the ninja's trailer.
function LuaCannonBall:updateTrailer( trailer )
	local s = SequenceAction()
			
	s:addAction( ColorAction( 0.4, InterpAction.Smooth, pool.get_colorf( 0.0, 0.0, 0.0, 1.0 ) ) )
			
	trailer:addAction( s )	
end
