------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Dart               --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local assets    = chunks.assets
local pool      = chunks.pool
local enemy     = chunks.enemy
local particles = chunks.particles
local actions   = chunks.actions
local textures  = chunks.textures
local sound     = chunks.sound
local utility   = chunks.utility

-- LuaDart --
LuaDart = chunks.class.inherit( chunks.enemy.LuaEnemy )()

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaDart )

-- The created method.
function LuaDart:created( owner )
	-- Initialize super.
	enemy.LuaEnemy.created( self, owner )
	
	-- Initialize owner.
	owner.label = "dart"
	
	-- Thrower.
	do
		local actor = owner:spawn( CollidableActor() )
		
		textures.setTexture( actor, assets.textures.dart )

		self.visuals.dart = actor
	end
	
	self:setVisual( "dart" )
end

-- The collided event.
function LuaDart:onCollided( other, is_bonus )
	local game    = self.owner:getGame()
	local ninja   = game.lua.ninja
	local spawner = is_bonus and particles.spawn_uber or particles.spawn_normal
	
	-- Spawn particles.
	do
		spawner( self.owner:getGame().lua.root, 
				 assets.textures.particles_feather, 
				 self.visual:getWorldLocation() )
	end
	
	self.owner:clearActions()
	actions.shrink_and_kill( self.visual )
	
	-- Play sound.
	sound.play_random( assets.sounds.hit_dart )
end

-- The bonus setup method.
function LuaDart:bonus( ninja )
	local offset      = ninja.lua.sides.left.offset
	local ldist       = math.abs( ninja.location.x - offset.x )
	local rdist       = math.abs( ninja.location.x + offset.x )
	local dir         = ldist > rdist and -1 or 1
	local delay       = 0.5
	local sleep_delay = 0.1
	local x_offset    = 275.0
	local set_rot = function( inner, d )
		ninja.lua:setRotation( v3f( 0.0, ( d == -1 ) and 180.0 or 0.0, 0.0 ), 
		                       v3f( 0.0, ( d ==  1 ) and 180.0 or 0.0, 0.0 ) )
	end
	
	textures.setTexture( ninja.lua.visual, assets.textures.bonus_dart )
	
	-- Enlarge shield.
	ninja.lua.shield.lua:bonusStart( 2.3 )
	
	-- Actions.
	do
		local s = SequenceAction()
		
		for i = 1,6 do   
			local x_start  = ( i == 1 ) and ninja.location.x or x_offset * dir * -1
			local x_end    = ( i == 6 ) and math.abs( offset.x ) * dir or x_offset * dir
			local y_offset = ( i == 1 ) and ninja.location.y 
			                            or ( ( i == 6 ) and offset.y 
			                            or offset.y + ( i - 1 ) * 70.0 ) 
		
			-- Update rotation.
			s:addAction( LuaAction( 0.0, set_rot, dir ) )
			
			-- Snap to the new location.
			s:addAction( MoveAction( 0.0, InterpAction.Lerp,
		                             pool.get_v3f( x_start, y_offset, offset.z ) ) )
			
			-- Move to a new target location.
			s:addAction( MoveAction( delay, InterpAction.Lerp,
		                             pool.get_v3f( x_end, y_offset, offset.z ) ) )
			
			-- Add a slight delay.
			if i < 6 then
				s:addAction( SleepAction( sleep_delay ) )
			end
			
			dir = dir * -1
		end
		
		s:addAction( LuaMemberAction( 0.0, "bonusStop" ) )	
		
		ninja:addAction( s )
	end
	
	-- Play sounds.
	sound.play( assets.sounds.bonus_trigger_dart )
	sound.play( assets.sounds.bonus_loop_dart )
	
	return delay * 6 + sleep_delay * 5
end

-- Update the ninja's trailer.
function LuaDart:updateTrailer( trailer )
	local s = SequenceAction()
			
	s:addAction( ColorAction( 0.4, InterpAction.Smooth, pool.get_colorf( 96.0 / 255.0, 250.0 / 255.0, 54.0 / 255.0, 1.0 ) ) )
			
	trailer:addAction( s )	
end
