------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Dragon             --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local assets    = chunks.assets
local pool      = chunks.pool
local enemy     = chunks.enemy
local particles = chunks.particles
local actions   = chunks.actions
local utility   = chunks.utility
local textures  = chunks.textures
local sound     = chunks.sound

-- LuaDragon --
LuaDragon = chunks.class.inherit( chunks.enemy.LuaEnemy )()

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaDragon )

-- The created method.
function LuaDragon:created( owner )
	-- Initialize super.
	enemy.LuaEnemy.created( self, owner )
	
	-- Initialize owner.
	owner.label = "dragon"
end

-- Setup a dragon sequence.
function LuaDragon:setup( fireball_layer, fireball_particle_layer, delay )
	local owner            = self.owner
	local hdist            = 210.0
	local fireball_spawner = nil
	local dragon           = nil
	
	-- The dragon head.
	do
		local actor = owner:spawn( SpriteActor() )
		
		textures.setTexture( actor, assets.textures.dragon )
		
		-- Initial location.
		actor.location = pool.get_v3f( -320.0, 140.0, 0.0 )
		actor.rotation = pool.get_v3f( 0.0, 0.0, -20.0 )
		
		-- Actions.
		do
			local s = SequenceAction()
			
			s:addAction( MoveOffsetAction( delay * 0.45, InterpAction.Smooth, pool.get_v3f( hdist, 0.0, 0.0 ) ) )
			s:addAction( SleepAction( delay * 0.1 ) )
			s:addAction( MoveOffsetAction( delay * 0.45, InterpAction.Smooth, pool.get_v3f( -hdist, 0.0, 0.0 ) ) )
			
			actor:addAction( s )
		end
		
		-- Spawn the fireball spawner.
		fireball_spawner = actor:spawn( ActorBase() )
		
		dragon = actor
	end
	
	-- The dragon wing.
	do
		local actor = owner:spawn( SpriteActor() )
		
		textures.setTexture( actor, assets.textures.dragon_wing )
		
		-- Initial location.
		actor.location = pool.get_v3f( -350.0, 30.0, 0.0 )
		
		-- Actions.
		do
			local s     = SequenceAction()
			local split = SplitAction()
			local s1    = SequenceAction()
			local s2    = SequenceAction()
			local s3    = SequenceAction()
			
			s1:addAction( MoveOffsetAction( delay * 0.5, InterpAction.Smooth, pool.get_v3f( hdist, 0.0, 0.0 ) ) )
			s1:addAction( MoveOffsetAction( delay * 0.5, InterpAction.Smooth, pool.get_v3f( -hdist, 0.0, 0.0 ) ) )
			
			for i = 1,5 do
				s2:addAction( ScaleOffsetAction( delay * 0.1, InterpAction.Smooth,
				              pool.get_v3f( -0.1, -0.1, 0.0 ) ) )
				s2:addAction( ScaleOffsetAction( delay * 0.1, InterpAction.Smooth,
				              pool.get_v3f( 0.1, 0.1, 0.0 ) ) )
				              
				s3:addAction( RotateOffsetAction( delay * 0.1, InterpAction.Smooth,
				              pool.get_v3f( 0.0, 0.0, -10.0 ) ) )
				s3:addAction( RotateOffsetAction( delay * 0.1, InterpAction.Smooth,
				              pool.get_v3f( 0.0, 0.0, 10.0 ) ) )              
			end              
			        
			split:addAction( s1 )
			split:addAction( s2 )
			split:addAction( s3 )
			s:addAction( split )
			
			actor:addAction( s )
		end
	end
	
	-- Spawn the fireballs.
	do
		local s = SequenceAction()
		local f = function( inner )
			local fireball       = fireball_layer:spawn( chunks.fireball.spawn() )
			local fireball_delay = 2.0
			
			-- Snap to the dragon's head.
			fireball.location = dragon.location + pool.get_v3f( -15.0, 0.0, 0.0 )
			fireball.scale    = pool.get_v3f( 0.0, 0.0, 1.0 )
			
			-- Spawn smoke trailer.
			local spawn_particles = function( o )
				-- Particles.
				local system = fireball_particle_layer:spawn( ParticleSystem() )
				
				-- Set explosion parameters.
				system.location                  = fireball.location
				system.texture                   = assets.textures.particles_fireball.path
				system.particle_size             = assets.textures.particles_fireball.size
				system.explosion                 = true
				system.lifetime                  = fireball_delay
				system.lifetime_range            = pool.get_v2f( 0.9, 1.0 )
				system.particles_per_spawn_range = pool.get_v2i( 6, 8 )
				system.spawn_frequency_range     = pool.get_v2f( 0.03, 0.04 )
				system.speed_range               = pool.get_v2f( 100, 140.0 )
				system.initial_rotation_range    = pool.get_v2f( 0.0, 360.0 )
				system.rotation_speed_range      = pool.get_v2f( 120.0, 150.0 )
				
				-- scale transitions
				system:addScaleRange( pool.get_v2f( 0.8, 1.0 ) )
				system:addScaleRange( pool.get_v2f( 0.4, 0.6 ) )
				system:addScaleRange( pool.get_v2f( 0.0, 0.0 ) )

				system:setReferenceActor( fireball:getGame() )
                system.follow_system = false

				-- Actions.
				do
					local s1    = SequenceAction()
					local split = SplitAction()
					local s2    = SequenceAction()
					local s3    = SequenceAction()
					
					s2:addAction( MoveOffsetAction( fireball_delay, InterpAction.Lerp,
								  pool.get_v3f( 260.0 * 1.6, -280.0 * 1.6, 0.0 ) ) )	  
					s3:addAction( ScaleAction( fireball_delay * 0.3, InterpAction.Smooth,
								  pool.get_v3f( 1.0, 1.0, 1.0 ) ) )
										
					split:addAction( s2 )
					split:addAction( s3 )
					s1:addAction( split )
					
					system:addAction( s1 )
				end
				
				-- Tell the fireball.
				fireball.lua.particles = system
				
				-- Play spit sound.
				sound.play_random( assets.sounds.dragon_spit )
			end
			
			-- Actions.
			do
				local s1    = SequenceAction()
				local split = SplitAction()
				local s2    = SequenceAction()
				local s3    = SequenceAction()
				local s4    = SequenceAction()
				
				s2:addAction( MoveOffsetAction( fireball_delay, InterpAction.Lerp,
				              pool.get_v3f( 260.0 * 1.6, -280.0 * 1.6, 0.0 ) ) )
				s3:addAction( ScaleAction( fireball_delay * 0.3, InterpAction.Smooth,
				              pool.get_v3f( 1.0, 1.0, 1.0 ) ) )
				s3:addAction( SleepAction( fireball_delay * 0.6 ) )
				s3:addAction( ScaleAction( fireball_delay * 0.1, InterpAction.Smooth,
				              pool.get_v3f( 0.0, 0.0, 1.0 ) ) )
				s4:addAction( SleepAction( fireball_delay * 0.15 ) )
				s4:addAction( LuaAction( 0.0, spawn_particles ) )
				
				split:addAction( s2 )
				split:addAction( s3 )
				split:addAction( s4 )
				s1:addAction( split )
				
				fireball:addAction( s1 )
			end
		end
		
		s:addAction( SleepAction( delay * 0.5 ) )
		s:addAction( LuaAction( 0.0, f ) )
		
		fireball_spawner:addAction( s )
	end	

	-- Play warning sound.
	sound.play_random( assets.sounds.dragon_enter )
end

-- Setup easy sequence.
function LuaDragon:easy( fireball_layer, fireball_particle_layer )
	self:setup( fireball_layer, fireball_particle_layer, 4.0 )
end

-- Setup hard sequence.
function LuaDragon:hard( fireball_layer, fireball_particle_layer ) 
	self:setup( fireball_layer, fireball_particle_layer, 2.6 )
end

