------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Manifest           --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local class = require( "shared.class" )

-- Seed random number generator.
math.randomseed( os.time() )
math.random(); math.random(); math.random()

-- globals --
instance = nil

-- LuaEngine --
LuaEngine = class.new
{
	platform   = nil,
	chunks     = nil,
	paths      = nil,
	product    = "",
	theme      = "",
    audio      = nil,
    default_language_enabled = true,
    locale_language = "en",
    locale_font_scale = 1.0,
    m_startup_tasks_complete = false,
    m_tasks = nil,
    m_async_handle = nil,
    m_last_suspend_time = nil,
    m_audio_initialized = false,
}

-- The created method.
function LuaEngine:created( owner )	
	-- Create the tables.
	self.platform = {}
	self.chunks = {}

	-- Figure out the device type.
	do
		local info     = DeviceInfo.get()
		local platform = info:query( "platform.type" )

		-- ios
		if platform == "ios" then
			local display_scale = info:query( "display.scale" )
			
			if display_scale then 
			    self.platform.display_scale = display_scale
			end

			self.platform.device = "iphone"

			if info:is( "ios.ipad_ui_idiom" ) then
				self.platform.device = "ipad"
            elseif display_scale and display_scale > 1.0 then
                self.platform.device = "retina"
			else
				self.platform.device = "iphone"
			end
		elseif platform == "mac" then
			self.platform.device = "ipad"
		elseif platform == "android" then
			self.platform.device = "android"
		end

		print( "device: " .. self.platform.device )
	end
	
	-- Get device language
	do
		local info 	= DeviceInfo.get()
		local language = info:query( "platform.language" )

		if language == "en" then
			self.default_language_enabled = true
		else
			self.default_language_enabled = false
		end
		
		if language == "ja" or language == "ko" or string.sub(language,1,string.len("zh")) == "zh" then
		    self.locale_font_scale = 0.7
		else
		    self.locale_font_scale = 0.9
		end
		
		self.locale_language = language
	end

	-- Load device-specific constants.
	do
		self.platform.constants = require( "platform." .. self.platform.device .. ".constants" )
		
		owner.delta_time_cap = self.platform.constants.delta_time_cap
	end

    -- Setup audio.
    do
        self.audio = {}
        self.audio.sound                 = Audio.get():createSoundPool()
        self.audio.music                 = createNinJumpPlaylist()
        self.audio.music.loop_last_track = true
    end

	-- Audio settings.
    Audio.get():getSoundPoolGroup().volume = GameOptions.get().sound_on and 1.0 or 0.0
    Audio.get():getPlaylistGroup().volume  = GameOptions.get().music_on and 1.0 or 0.0

    Audio.get():restoreState()
    
	-- Randomly set theme on startup.
	if self.product == "deluxe" then
		local themes = nil

        local iap_util = require("shared.iap")
        if not iap_util.adBuyoutPurchased() then
			themes = { "jungle", "pirates", "castle" }
		else
			themes = { "classic", "jungle", "pirates", "castle" }
		end
		
		self.theme = themes[ math.random( 1, #themes ) ]
	else
		self.theme = "classic"
	end
	
	-- Make ourself global.
	instance = self
	
	local startup = function()
		-- Setup paths.
		self.paths = 
		{
			scripts_shared  = "shared",
			textures_shared = "?://texture/shared/",
			sounds_shared   = "?://sound/shared/"
		}
	
		local precache = require( "shared.precache" )
		local assets   = require( "shared.assets" )
		
		precache.load( assets )
        
        
        -- For now, spawn the classic theme.
        self:spawnMenuGame()

        -- Fire the ready event.
        self.chunks.events.onGameReady()
    end

    --Nixing the load screen - it gets corrupted sometimes and the wait isnt that big of a deal.
    --if self.platform.device == "android" then
    --    self.platform.constants.loading( self, startup )
    --else
    --    startup()
    --end
    
    
    
    startup()
    
    
end  --End: Instance Created

-- The engine is suspended.
function LuaEngine:suspend()
    if self.game and self.game.lua and self.game.lua.onPause then
        self.game.lua:onPause()
    end
    self.m_last_suspend_time = timer.getTime()
    
    self:queueLocalNotifications()
end

function LuaEngine:willResignActive()
    --This is called when the swipe-down menu is opened. When else?
    if self.game and self.game.lua and self.game.lua.onPause then
        self.game.lua:onPause()
    end
end

-- The engine is resumed.
function LuaEngine:resume()
    --[[
    local resume_time = timer.getTime()
   
    if((resume_time - self.m_last_suspend_time) > 600) then
        self.chunks.analytics.reportSessionStarted()
    end
    ]]
end 

function LuaEngine:pauseIfNecessary(owner)
	if self.game and self.game.lua and self.game.lua.onPause then
        self.game.lua:onPause()
    end
end

function LuaEngine:queueLocalNotifications()

    local service = NotificationService.get()
    if service then
        
        service:cancelScheduledLocalNotifications()
        
        local details = self.chunks.social.getSocialDetails()
        local product_name = details.product_name
        local seconds_in_day = 86400
        
        do
            local notif = service:newLocalNotification()
            notif.action = product_name
            notif.message = lstring("notification_1")
            notif.category = "reminder"
            service:scheduleLocalNotificationIn(1 * seconds_in_day, notif)
        end
        
        do       
            local notif = service:newLocalNotification()
            notif.action = product_name
            notif.message = lstring("notification_2")
            notif.category = "reminder"
            service:scheduleLocalNotificationIn(3 * seconds_in_day, notif)
        end
        
        do       
            local notif = service:newLocalNotification()
            notif.action = product_name
            notif.message = lstring("notification_3")
            notif.category = "reminder"
            service:scheduleLocalNotificationIn(7 * seconds_in_day, notif)
        end
        
        do       
            local notif = service:newLocalNotification()
            notif.action = product_name
            notif.message = lstring("notification_4")
            notif.category = "reminder"
            service:scheduleLocalNotificationIn(14 * seconds_in_day, notif)
        end
    end
end

-- Spawn a menu game.
function LuaEngine:spawnMenuGame()
	-- Unload game assets.
	if self.chunks.precache ~= nil then
		self.chunks.precache.unload( self.chunks.assets )
	end
	
	-- Setup paths.
	self.paths = 
	{
		scripts_shared   = "shared",
		scripts_product  = "products." .. self.product,
		textures_shared  = "?://texture/shared/",
		textures_product = "?://texture/products/" .. self.product .. "/",
		music_shared     = "?://music/shared/",
		sounds_shared    = "?://sound/shared/"
	}
	
	-- Clear all preloaded modules.
	for k,v in pairs( package.loaded ) do
		package.loaded[ k ] = nil
	end
	
	-- Collect garbage.
	collectgarbage( "collect" )
	
	-- Load the chunks.
	require( "products." .. self.product .. ".chunks" )
	
	-- Overrides.
	if self.platform.constants.menu_overrides ~= nil then
		self.platform.constants.menu_overrides()
	end

	-- Spawn the menu game.
    self.game = self.owner:getRoot():spawn( self.chunks.game.spawn() )
    
    --self.chunks.analytics.reportSessionStarted()
end

-- Spawn a game.
function LuaEngine:spawnMainGame( theme, replay )
	replay = replay or false

	-- Unload menu assets.
	if replay == false and self.chunks ~= nil then
		self.chunks.precache.unload( self.chunks.assets )
	end
	
	-- Setup the theme.
	self.theme = theme
	
	-- Setup paths.
	self.paths = 
	{
		scripts_shared   = "shared",
		scripts_theme    = "themes." .. theme,
		textures_shared  = "?://texture/shared/",
		textures_theme   = "?://texture/themes/" .. theme .. "/",
		meshes           = "?://mesh/themes/" .. theme .. "/",
		music_theme      = "?://music/themes/" .. theme .. "/",
		sounds_shared    = "?://sound/shared/",
		sounds_theme     = "?://sound/themes/" .. theme .. "/"
	}
	
	-- Clear all preloaded modules.
	for k,v in pairs( package.loaded ) do
		package.loaded[ k ] = nil
	end

	-- Load the chunks.
	require( "themes." .. theme .. ".chunks" )
	
	-- Overrides.
	if self.platform.constants.game_overrides ~= nil then
		self.platform.constants.game_overrides()
	end
	
	-- Precache.
	if replay == false then
		self.chunks.precache.load( self.chunks.assets )
	end
	
	-- Collect garbage.
	collectgarbage( "collect" )
	
	-- Spawn the game.
	do
        local game = self.owner:getRoot():spawn( self.chunks.game.spawn() )
	
		game.lua:intro( replay )
        self.game = game
	end
end

-- Bind the engine.
function bindEngine( owner, product )
	local engine = LuaEngine:new( owner )
	
	-- Set the product.
	engine.lua.product = product
end

-- Hack for anchors.
_G[ "make_anchor" ] = function( source, target, offset )
    return { source, target, v3f( offset ) }
end


-- Startup Tasks --
function LuaEngine:_queueTask(task)
    local count = #self.m_tasks
    table.insert(self.m_tasks, count + 1, task)
end

function LuaEngine:_runNextTask()

    local count = #self.m_tasks
    if count > 0 then
        func = self.m_tasks[1]
        table.remove(self.m_tasks, 1)
        func()
    end

end


function LuaEngine:doStartupTasks()
    -- Only run the startup tasks once.
    if self.m_startup_tasks_complete == true then
        return
    end

    self.m_startup_tasks_complete = true
    self.m_tasks = {}
    
    if not GameService.get():zeb() then		-- no swrve in zeb builds
        local swrve_start_session_task = function()    
            self.chunks.swrve.initiateUserSession()
            self.chunks.swrve.populateDefaultResources()
            self:_runNextTask()
        end
        
        self:_queueTask(swrve_start_session_task)
    end
    
    -- Show IAP Warning
    if not UserData.instance():getSeenIAPWarning() and not GameService.get():zeb() then		-- IAP warning is irrelevant for zeb builds
        local iap_task = function()
            print("Running IAP warning task")

            local ui = PlatformUIService.get()
            if ui ~= nil then
                local title = lstring("iap_warning_title")
				
				local info     = DeviceInfo.get()
				local platform = info:query( "platform.type" )
				
				local body = lstring("iap_warning_body")
				
				if platform == "android" then
					body = lstring("iap_warning_body_android")
				end

                local details = self.chunks.social.getSocialDetails()
                local product_name = details and details.product_name or lstring.format("%1"):arg(1,"This")

                body:arg(1, product_name)
                local okay = lstring("ok")
                local callback = function(handle, responseCode, response)
                    print("IAP warning task completed: " .. tostring(responseCode))
                    UserData.instance():setSeenIAPWarning(true)
                    self.m_async_handle = nil
                    self:_runNextTask()
                end
                
                self.m_async_handle = ui:showDialog(title, body, okay, callback)
            else
                self:_runNextTask()
            end
        end
        self:_queueTask(iap_task)
    end

    -- App Update Task
    do
        local update_task = function()
            print("Running app update check task")
            local ui = PlatformUIService.get()
            if ui ~= nil then
                local title = lstring("application_update_message_1")
                local message = lstring("application_update_message_2");
                local yes_button = lstring("application_update_yes");
                local no_button = lstring("application_update_no");
                local later_button = lstring("application_update_later");

                local callback = function(handle, responseCode, response)
                    print("App update check task completed: " .. tostring(responseCode))
                    self.m_async_handle = nil
                    self:_runNextTask()
                end

                self.m_async_handle = ui:promptForApplicationUpdate(title, message, yes_button, no_button, later_button, callback)
            else
                self:_runNextTask()
            end
        end
        self:_queueTask(update_task)
    end

    -- GameCenter Login
    do
        local gc = GameCenterService.get()
        if gc then
            local gc_task = function()
                print("Running GameCenter login task")
                if gc ~= nil then
                    local callback = function(handle, responseCode, response)
                        print("GameCenter login task completed: " .. tostring(responseCode))

                        self.m_async_handle = nil
                        self:_runNextTask()
                    end

                    self.m_async_handle = gc:login(callback)
                else
                    self:_runNextTask()
                end
            end
            self:_queueTask(gc_task)
        end
    end

    self:_runNextTask()
end

function LuaEngine:onStartupInterstitialDismissed()
    self.m_audio_initialized = true
    self:playMenuMusic()
end

function LuaEngine:playMenuMusic()
    if self.chunks and self.chunks.assets and self.chunks.assets.music and self.m_audio_initialized == true then
        -- Start music.
        self.chunks.music.play( self.chunks.assets.music.main )
    end
end