------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Fireball           --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local assets    = chunks.assets
local pool      = chunks.pool
local enemy     = chunks.enemy
local particles = chunks.particles
local actions   = chunks.actions
local textures  = chunks.textures
local sound     = chunks.sound
local utility   = chunks.utility

-- LuaFireball --
LuaFireball = chunks.class.inherit( chunks.enemy.LuaEnemy )
{
	particles = nil
}

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaFireball )

-- The created method.
function LuaFireball:created( owner )
	-- Initialize super.
	enemy.LuaEnemy.created( self, owner )
	
	-- Initialize owner.
	owner.label = "fireball"
	
	-- Thrower.
	do
		local actor = owner:spawn( CollidableActor() )
		
		textures.setTexture( actor, assets.textures.fireball )

		self.visuals.fireball = actor
	end
	
	self:setVisual( "fireball" )
end

-- Bonus meter override.
function LuaFireball:bonusmeterOverride( actor )
    actor.scale = pool.get_v3f( 0.6, 0.6, 1.0 )
    
    textures.setTexture( actor, assets.textures.fireball )
end

-- The collided event.
function LuaFireball:onCollided( other, is_bonus )
	local game    = self.owner:getGame()
	local ninja   = game.lua.ninja
	local spawner = is_bonus and particles.spawn_uber or particles.spawn_normal
	
	-- Spawn particles.
	do
		spawner( self.owner:getGame().lua.root, 
				 assets.textures.particles_fireball, 
				 self.visual:getWorldLocation() )
	end
	
	-- Kill the particles.
	if self.particles then
		self.particles.lifetime = 0.0
		self.particles          = nil
	end
	
	self.owner:clearActions()
	actions.shrink_and_kill( self.visual )
	
	-- Play sound.
	sound.play_random( assets.sounds.hit_fireball )
end

-- The bonus setup method.
function LuaFireball:bonus( ninja )
	local owner   = self.owner
	local game    = owner:getGame()
	local offset  = ninja.lua.sides.left.offset 
	local dir     = ( ninja:getWorldLocation().x < 0.0 ) and -1 or 1
	local start   = v3f( math.abs( offset.x ) * dir, -400.0, ninja.location.z )
	local height  = 810.0
	local delay   = 8.0
	local set_rot = function( inner, dir )
		local r = v3f( 0.0, ( dir == 1 ) and 180.0 or 0.0, 0.0 )
		
		ninja.lua:setRotation( r, r )
	end
	local spawn_flames = function( owner, dir )
		local system = owner:spawn( ParticleSystem() )
		
		-- Set explosion parameters.
		system.location                  = pool.get_v3f( 18.0, -18.0, dir )
		system.rotation                  = pool.get_v3f( 0.0, 0.0, -32.0 )
		system.texture                   = assets.textures.particles_fireball.path
		system.particle_size             = assets.textures.particles_fireball.size
		system.explosion                 = true
		system.lifetime                  = delay * 0.4
		system.shape                     = ParticleSystem.Cone
		system.cone_angle_range          = pool.get_v2f( 12.0, -12.0 )
		system.lifetime_range            = pool.get_v2f( 1.2, 1.3 )
		system.particles_per_spawn_range = pool.get_v2i( 6, 8 )
		system.spawn_frequency_range     = pool.get_v2f( 0.03, 0.04 )
		system.speed_range               = pool.get_v2f( 250, 350.0 )
		system.initial_rotation_range    = pool.get_v2f( 0.0, 360.0 )
		system.rotation_speed_range      = pool.get_v2f( 120.0, 150.0 )
		system.follow_system             = true
		system.max_particles             = 300
		
		-- scale transitions
		system:addScaleRange( pool.get_v2f( 0.0, 0.0 ) )
		system:addScaleRange( pool.get_v2f( 0.4, 0.6 ) )
		system:addScaleRange( pool.get_v2f( 0.6, 0.8 ) )
		system:addScaleRange( pool.get_v2f( 0.4, 0.6 ) )
		system:addScaleRange( pool.get_v2f( 0.0, 0.0 ) )
		
		return system
	end
	local flames = spawn_flames( ninja.lua.visual, dir )
	
	-- Set texture.
	textures.setTexture( ninja.lua.visual, assets.textures.bonus_dragon )
	
	-- Enlarge shield.
	ninja.lua.shield.lua:bonusStart( 3.8 )
	
	-- Initial values.
	ninja.location = start
	set_rot( ninja, dir )
	
	-- Actions.
	do
		local s = SequenceAction()
		
		-- Move up..
		s:addAction( MoveOffsetAction( delay * 0.5, InterpAction.Lerp,
		             pool.get_v3f( 0.0, height, 0.0 ) ) )
		             
		-- Switch sides and rotate.
		s:addAction( LuaAction( 0.0, function( inner )
			-- Set flames owner to be the scene.
			do
				local world = flames:getWorldLocation()
				
				flames:setOwner( game:getScene() )
				flames.location = world
				flames.rotation = pool.get_v3f( 0.0, ( dir == -1 ) and 0.0 or 180.0, flames.rotation.z )
			end
			
			-- Change direction.
			dir = dir * -1
			
			-- Spawn new flames.
			flames = spawn_flames( ninja.lua.visual, dir )
			
			-- Update rotation, location.
			do
				ninja.location = pool.get_v3f( math.abs( offset.x ) * dir, start.y + height, start.z )
			
				set_rot( ninja, dir )
			end
		end ) )
		
		-- Move down.
		s:addAction( MoveOffsetAction( delay * 0.5, InterpAction.Lerp,
		             pool.get_v3f( 0.0, -height, 0.0 ) ) )
		             
		-- Re-owner flames.
		s:addAction( LuaAction( 0.0, function( inner )
			flames:destroy()
			flames = nil
		end ) )
		
		-- Done.
		s:addAction( LuaMemberAction( 0.0, "bonusStop" ) )	
		
		-- Spawn puff of smoke.
		s:addAction( LuaAction( 0.0, function( inner )
			particles.spawn_big( ninja:getGame().lua.root, 
								 assets.textures.particles_smoke, 
								 ninja:getWorldLocation() )
		end ) )
	
		ninja:addAction( s )
	end
	
	-- Play bonus sound.
	sound.play( assets.sounds.bonus_dragon )
	
	return delay
end

-- Update the ninja's trailer.
function LuaFireball:updateTrailer( trailer )
	local s = SequenceAction()
			
	s:addAction( ColorAction( 0.4, InterpAction.Smooth, 
	             pool.get_colorf( 255.0 / 255.0, 157.0 / 255.0, 0.0 / 255.0, 1.0 ) ) )
	
	trailer:addAction( s )	
end
