------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Frog               --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local assets    = chunks.assets
local pool      = chunks.pool
local enemy     = chunks.enemy
local particles = chunks.particles
local actions   = chunks.actions
local utility   = chunks.utility
local textures  = chunks.textures
local sound     = chunks.sound
local effects   = chunks.effects

-- LuaFrog --
LuaFrog = chunks.class.inherit( chunks.enemy.LuaEnemy )
{
	offset = -120.0
}

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaFrog )

-- The created method.
function LuaFrog:created( owner )
	-- Initialize super.
	enemy.LuaEnemy.created( self, owner )
	
	-- Initialize owner.
	owner.label    = "frog"
	owner.location = pool.get_v3f( self.offset, 0.0, 0.0 )
	
	-- Thrower.
	do
		local actor = owner:spawn( CollidableActor() )
		
		textures.setTexture( actor, assets.textures.frog )
		
		actor.scale = pool.get_v3f( 1.2, 1.2, 1.0 )

		self.visuals.frog = actor
	end
	
	self:setVisual( "frog" )
end

-- The collided event.
function LuaFrog:onCollided( other, is_bonus )
	local owner   = self.owner
	local game    = owner:getGame()
	local ninja   = game.lua.ninja
	local spawner = is_bonus and particles.spawn_uber or particles.spawn_normal
	
	-- Spawn particles.
	do
		local system = spawner( game.lua.root, 
		                        assets.textures.particles_trippy, 
		                        self.visual:getWorldLocation() )
		         
		system:addColorRange( pool.get_colorf( 1.0, 1.0, 1.0, 1.0 ) )
		system:addColorRange( pool.get_colorf( 1.0, 1.0, 1.0, 1.0 ) )
		system:addColorRange( pool.get_colorf( 1.0, 1.0, 1.0, 0.0 ) )
		
		system.rotation_speed_range = pool.get_v2f( 360.0, 500.0 )
	end
	
	-- Spawn slice.
	do
		local flash   = game.lua.root:spawn( SpriteActor() )
		local degrees = 360.0
		
		flash.location = self.visual:getWorldLocation()
		flash.color    = pool.get_colorf( 1.0, 1.0, 1.0, 0.9 )
		flash.scale    = pool.get_v3f( 0.5, 0.5, 1.0 )
		
		textures.setTexture( flash, assets.textures.effects_trippy )
		
		-- Actions.
		do
			local s     = SequenceAction()
			local s2    = SequenceAction()
			local split = SplitAction()
			local delay = is_bonus and 0.8 or 0.5
			
			s2:addAction( ScaleAction( delay / 2.0, InterpAction.Smooth, pool.get_v3f( 1.5, 1.5, 1.0 ) ) )
			s2:addAction( ScaleAction( delay / 2.0, InterpAction.Smooth, pool.get_v3f( 0.5, 0.5, 1.0 ) ) )
			split:addAction( s2 )
			split:addAction( AlphaAction( delay, InterpAction.Smooth, 0.0 ) )
			split:addAction( RotateOffsetAction( delay, InterpAction.Lerp, pool.get_v3f( 0.0, 0.0, degrees ) ) )
			s:addAction( split )
			s:addAction( DestroyAction( 0.0 ) )
			
			flash:addAction( s )
		end
	end
	
	self.owner:clearActions()
	actions.shrink_and_kill( self.visual )
	
	-- Play sound.
	sound.play_random( assets.sounds.hit_frog )
end

-- Setup easy sequence.
function LuaFrog:easy()
	local actor = self.owner
	
	-- Owner actions
	do
		local s      = SequenceAction()
		local l      = LoopAction()
		local split1 = SplitAction()
		local split2 = SplitAction()
		local s1     = SequenceAction()
		local s2     = SequenceAction()
		local delay1 = utility.frandom( 1.0, 1.6 )
		local delay2 = utility.frandom( 1.0, 1.6 )
		local vdist  = 15.0
		local f      = function( inner, rot )
			self.visual.rotation = pool.get_v3f( 0.0, rot, 0.0 )
		end
		
		-- Initial sleep, setup rotation.
		s:addAction( SleepAction( utility.frandom( 0.4, 0.6 ) ) )
		l:addAction( LuaAction( 0.0, f, 0.0 ) )
		
		-- Move across.
		split1:addAction( MoveOffsetAction( delay1, 
		                  InterpAction.Lerp, pool.get_v3f( self.offset * -2, 0.0, 0.0 ) ) )    
		s1:addAction( MoveOffsetAction( delay1 / 2.0, 
		              InterpAction.Smooth, pool.get_v3f( 0.0, -vdist, 0.0 ) ) )    
		s1:addAction( MoveOffsetAction( delay1 / 2.0, 
		              InterpAction.Smooth, pool.get_v3f( 0.0, vdist, 0.0 ) ) )        
		split1:addAction( s1 )
		l:addAction( split1 )
		
		-- Sleep for a second.
		l:addAction( SleepAction( utility.frandom( 0.05, 0.15 ) ) )
		l:addAction( LuaAction( 0.0, f, 180.0 ) )
		
		-- Move back.
		split2:addAction( MoveOffsetAction( delay2, 
		                  InterpAction.Lerp, pool.get_v3f( self.offset * 2, 0.0, 0.0 ) ) )    
		s2:addAction( MoveOffsetAction( delay2 / 2.0, 
		              InterpAction.Smooth, pool.get_v3f( 0.0, -vdist, 0.0 ) ) )    
		s2:addAction( MoveOffsetAction( delay2 / 2.0, 
		              InterpAction.Smooth, pool.get_v3f( 0.0, vdist, 0.0 ) ) )
		split2:addAction( s2 )
		l:addAction( split2 )
		
		-- Sleep for a second.
		l:addAction( SleepAction( utility.frandom( 0.05, 0.15 ) ) )
		s:addAction( l )
		
		actor:addAction( s )
	end	
end

-- Setup easy sequence.
function LuaFrog:hard()
	local actor = self.owner
	
	-- Owner actions
	do
		local s      = SequenceAction()
		local l      = LoopAction()
		local split1 = SplitAction()
		local split2 = SplitAction()
		local s1     = SequenceAction()
		local s2     = SequenceAction()
		local delay1 = utility.frandom( 0.8, 1.0 )
		local delay2 = utility.frandom( 1.0, 1.6 )
		local vdist  = 15.0
		local f      = function( inner, rot )
			self.visual.rotation = pool.get_v3f( 0.0, rot, 0.0 )
		end
		
		-- Initial sleep, setup rotation.
		s:addAction( SleepAction( utility.frandom( 0.65, 0.65 ) ) )
		l:addAction( LuaAction( 0.0, f, 0.0 ) )
		
		-- Move across.
		split1:addAction( MoveOffsetAction( delay1, 
		                  InterpAction.Lerp, pool.get_v3f( self.offset * -2, 0.0, 0.0 ) ) )    
		s1:addAction( MoveOffsetAction( delay1 / 2.0, 
		              InterpAction.Smooth, pool.get_v3f( 0.0, -vdist, 0.0 ) ) )    
		s1:addAction( MoveOffsetAction( delay1 / 2.0, 
		              InterpAction.Smooth, pool.get_v3f( 0.0, vdist, 0.0 ) ) )        
		split1:addAction( s1 )
		l:addAction( split1 )
		
		-- Sleep for a second.
		l:addAction( SleepAction( utility.frandom( 0.05, 0.15 ) ) )
		l:addAction( LuaAction( 0.0, f, 180.0 ) )
		
		-- Move back.
		split2:addAction( MoveOffsetAction( delay2, 
		                  InterpAction.Lerp, pool.get_v3f( self.offset * 2, 0.0, 0.0 ) ) )    
		s2:addAction( MoveOffsetAction( delay2 / 2.0, 
		              InterpAction.Smooth, pool.get_v3f( 0.0, -vdist, 0.0 ) ) )    
		s2:addAction( MoveOffsetAction( delay2 / 2.0, 
		              InterpAction.Smooth, pool.get_v3f( 0.0, vdist, 0.0 ) ) )
		split2:addAction( s2 )
		l:addAction( split2 )
		
		-- Sleep for a second.
		l:addAction( SleepAction( utility.frandom( 0.05, 0.15 ) ) )
		s:addAction( l )
		
		actor:addAction( s )
	end	
end

-- The bonus setup method.
function LuaFrog:bonus( ninja )
	local left_offset = ninja.lua.sides.left.offset
	local offset      = v3f( left_offset.x + 18.0, left_offset.y, left_offset.z )
	local ldist       = math.abs( ninja.location.x - offset.x )
	local rdist       = math.abs( ninja.location.x + offset.x )
	local dir         = ldist > rdist and -1 or 1
	local delay       = 0.4
	local sleep_delay = 0.2
	local set_rot = function( d )
		local r = v3f( 0.0, ( d == 1 ) and 180.0 or 0.0, 0.0 )
		
		ninja.lua:setRotation( r, r )
	end
	local start = function( inner, d )
		set_rot( d )
		textures.setTexture( ninja.lua.visual, assets.textures.bonus_frog_blur )

		-- Spawn particles.
		do
			local system = ninja.lua.visual:spawn( ParticleSystem() )
			local angle  = ( dir == 1 ) and 180.0 or 0.0

			-- Set explosion parameters.
            system:setReferenceActor( ninja:getGame() )
            system.follow_system = false

			system.texture                   = assets.textures.bonus_frog_blur.path
			system.particle_size             = assets.textures.bonus_frog_blur.size
			system.shape                     = ParticleSystem.Cone
			system.cone_angle_range          = v2f( angle, angle )
			system.explosion                 = true
			system.lifetime                  = delay * 9.0 / 10.0
			system.lifetime_range            = v2f( delay, delay )
			system.particles_per_spawn_range = v2i( 1, 1 )
			system.spawn_frequency_range     = v2f( delay * 0.04, delay * 0.04 )

			-- color
			system:addColorRange( colorf( 1.0, 1.0, 1.0, 1.0 ) )
			system:addColorRange( colorf( 1.0, 1.0, 1.0, 0.0 ) )	
		end
		
		-- Play swoosh effect.
		sound.play( assets.sounds.bonus_frog_swoosh )
	end
	local stop = function( inner, d )
		set_rot( d )
		textures.setTexture( ninja.lua.visual, assets.textures.bonus_frog_still )
	end
	
	-- Enlarge shield.
	ninja.lua.shield.lua:bonusStart( 2.5 )
	
	-- Actions.
	do
		local s = SequenceAction()
		
		for i = 1,6 do
			-- Start.
			s:addAction( LuaAction( 0.0, start, dir ) )
			
			-- Horizontal movement.
			s:addAction( MoveAction( delay, InterpAction.Smooth, 
			                          pool.get_v3f( math.abs( offset.x ) * dir, offset.y, offset.z ) ) )
			
			-- Stop.
			s:addAction( LuaAction( 0.0, stop, dir ) )
			                          
			-- Sleeps.
			if i < 6 then
				s:addAction( SleepAction( sleep_delay ) )
			end
			
			dir = dir * -1
		end
		
		s:addAction( LuaMemberAction( 0.0, "bonusStop" ) )	
		
		ninja:addAction( s )
	end
	
	-- Play sounds.
	sound.play( assets.sounds.bonus_trigger_frog )
	
	return delay * 6 + sleep_delay * 5
end

-- Update the ninja's trailer.
function LuaFrog:updateTrailer( trailer )
	local s = SequenceAction()
			
	s:addAction( ColorAction( 0.4, InterpAction.Smooth, pool.get_colorf( 255.0 / 255.0, 0.0 / 255.0, 234.0 / 255.0, 1.0 ) ) )
	
	trailer:addAction( s )	
end

