------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- LuaMainGame        --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local constants = engine.platform.constants
local pool      = chunks.pool
local utility   = chunks.utility
local scenarios = chunks.scenarios
local assets    = chunks.assets
local events    = chunks.events
local music     = chunks.music
local textures = chunks.textures

-- LuaMainGame --
LuaMainGame = chunks.class.new
{
	-- actors --
	root = nil,
	
	-- walls --
	speed = -340.0,
	
	-- height --
	height_factor  = 10.0,
	height         = 0.0,
	best_height    = 0.0,
	spawned_banner = false,
	
	-- shield --
	shield_delay_range = v2f( 15.0, 20.0 ),
	shield_timer       = 0.0,
	
	-- scenarios --
	scenario_root  = nil,
	scenario_timer = 0.0,
	foliage_timer  = 0.0,
	
	-- bonus --
	bonus_timer      = 0.0,
	bonus_extra_time = 0.5,
	bonus_empty_time = 2.25,
	
	-- references --
	background = nil,
	walls      = nil,
	ninja      = nil,
	hud        = nil,
	
	-- pause menu, to remotely open shield store --
	pause_menu = nil,
	death_menu = nil,
	game_over = nil,
	
	-- incentivized starting boost related
	start_boost_in_progress = false,
	start_boost_speed = -1050,
	
	-- fuse video functionality
	ignore_next_pause_event = false
}

-- The spawner.
spawn = chunks.class.spawner( MainGame, LuaMainGame )

-- The created method.
function LuaMainGame:created( game )

	-- Fill shield meter to capacity
	local userdata = UserData.get()
	
	local max_shields = (UserData.get():getFreeShieldCapacity() + UserData.get():getShieldCapacity())
	UserData.get():setShields(max_shields)

	-- Spawn root.
	self.root = game:getScene():spawn( ActorBase() )

	-- Scene camera.
	do
		local camera = game:getScene():spawn( OrthographicCamera() )
		
		camera.world_size      = constants.scene.world_size
		camera.clipping_planes = constants.scene.clipping_planes
		camera.clear_color     = constants.clear_color
        camera.clear_color_on  = constants.clear_color_on
        camera.clear_depth_on  = constants.clear_depth_on
        camera.scene           = game:getScene()
	end
	
	-- Window camera.
	do
		local camera = game:getWindow():spawn( OrthographicCamera() )
		
		camera.world_size      = constants.window.world_size
		camera.clipping_planes = constants.window.clipping_planes
		camera.clear_color_on  = false
        camera.clear_depth_on  = false
        camera.scene           = game:getWindow()

        game:getWindow().camera = camera
	end
	
	-- Set best height.
	self.best_height = UserData.get():getHighScore( engine.theme )
	
	-- Setup scenario root.
	self.scenario_root = self.root:spawn( ActorBase() )
	--Debug stuff, TOTO delete me eventually
	--local actor = self.scenario_root:spawn( SpriteActor() )
	--textures.setTexture( actor, assets.textures.ninja_climbing )
	
	-- Spawn the background.
	self.background = self.root:spawn( chunks.background.spawn() )
	
	-- Spawn the walls.
	self.walls = self.root:spawn( chunks.walls.spawn() )
	
	-- Spawn the ninja.
	self.ninja = self.root:spawn( chunks.ninja.spawn() )
	
	-- Spawn the foreground.
	self.root:spawn( chunks.foreground.spawn() )
	
	-- Spawn the hud.
	self.hud = game:getWindow():spawn( chunks.hud.spawn() )

    -- Play Music
    local songs = { assets.music.intro, assets.music.loop }
    music.playList( songs )

	self.game_over = false
	self.start_boost_in_progress = false
	
	events.on_start_game()
end

-- The tick event.
function LuaMainGame:tick( game, delta_time )
	local adjusted = self.ninja.lua.bonus and delta_time * self.ninja.lua.bonus_factor or delta_time
	
	-- If we're not paused and the ninja isn't dying, update things.
	if game.paused == false and self.ninja.lua.dead == false then
		-- Update bonus (use unadjusted delta_time).
		self:updateBonus( delta_time )
		
		-- Update the height.
		self:updateHeight( adjusted )

		-- Update scenarios.
		self:updateScenarios( adjusted )
	end
end

function LuaMainGame:beginStartBoost()
    start_boost_in_progress = true
end

function LuaMainGame:endStartBoost()
    start_boost_in_progress = false
end

-- Update height.
function LuaMainGame:updateHeight( delta_time )
    if(start_boost_in_progress) then
        self.height = self.height + ( delta_time * math.abs( self.start_boost_speed ) 
			                  / self.height_factor )
    else
	    self.height = self.height + ( delta_time * math.abs( self.speed ) 
			                  / self.height_factor )
	end
	
	self.hud.lua.ticker.value = self.height
end

-- Update bonus.
function LuaMainGame:updateBonus( delta_time )
	-- Update bonus timer.
	if self.bonus_timer > 0.0 then
		self.bonus_timer = self.bonus_timer - delta_time
	end
end

-- Choses a scenario, based on the game's height.
function LuaMainGame:chooseScenario()
	local height       = self.height
	local transitions  = scenarios.transitions
	local size         = #transitions
	local allow_shield = self.shield_timer <= 0.0 and not self.ninja.lua.bonus 

	-- First, find the correct transition.
	for i = 1,size do
		local transition = transitions[ i ]

		-- Are we in the right transition?
		if height <= transition.height or i == size then
			-- Keep looping until we find one.
			while true do
				local prob = math.random()
				local s    = transition.scenarios[ math.random( #transition.scenarios ) ]
				
				-- Conditions:
				-- 1) The probability is within range.
				-- 2) Shields must be allowed if a shield is chosen.
				if prob <= s.prob and not ( s.f == scenarios.scenarios.shieldpack and not allow_shield ) then
					return s.f
				end
			end
		end
	end	
end

-- Spawn a scenario.  Returns the amount of
-- time to wait before spawning another scenario.
function LuaMainGame:spawnScenario()
	local game     = self.owner
	local root     = self.scenario_root:spawn( chunks.scenario.spawn() )
	local f        = nil
	local result   = nil
	
	-- Spawn new high score banner?
	if not self.spawned_banner and self.height > self.best_height and self.best_height > 0 then
		f                   = scenarios.scenarios.new_high_score
		self.spawned_banner = true
	-- Force empty scenarios during end of bonus?
	elseif self.bonus_timer > 0.0 and self.bonus_timer <= self.bonus_empty_time then
		f = scenarios.scenarios.empty
	-- Otherwise, randomly choose one.
	else
		f = self:chooseScenario()
	end
	
	-- If a shieldpack was chosen, update timer.
	if f == scenarios.scenarios.shieldpack then
		self.shield_timer = utility.frandom( self.shield_delay_range.x, 
			                                 self.shield_delay_range.y )
	end
	
	-- Spawn the scenario.
	local result = f( root )
	
	-- Randomly mirror the scenario if it's not a new high score banner.
	if f ~= scenarios.scenarios.new_high_score and math.random() < 0.5 then
		root.lua:mirror()
	end
	
	-- Setup cleanup actions.
	do
		local s = SequenceAction()
		
		s:addAction( DestroyAction( result.lifetime ) )
		
		root:addAction( s )
	end
	
	return result.delay
end

function LuaMainGame:spawnFoliage()
	local foliage = scenarios.scenarios.foliage
	
	-- No foliage?
	if foliage == nil then
		return 0.0
	end
	
	local game   = self.owner
	local root   = self.scenario_root:spawn( chunks.scenario.spawn() )
	local result = foliage( root )

	-- Randomly mirror the foliage.
	if math.random() < 0.5 then
		root.lua:mirror()
	end
	
	-- Setup cleanup actions.
	do
		local s = SequenceAction()
		
		s:addAction( DestroyAction( result.lifetime ) )
		
		root:addAction( s )
	end
	
	return result.delay
end

-- Scenarios.
function LuaMainGame:updateScenarios( delta_time )
	-- Update shield timing information.
	if self.shield_timer > 0.0 then
		self.shield_timer = self.shield_timer - delta_time
	end
	
	-- See if we need to spawn another foliage.
	do
		self.foliage_timer = self.foliage_timer - delta_time
		
		if self.foliage_timer <= 0.0 then
			self.foliage_timer = self:spawnFoliage()
		end
	end
		
	-- See if we need to spawn another scenario.
	do
		self.scenario_timer = self.scenario_timer - delta_time
		
		if self.scenario_timer <= 0.0 then
			self.scenario_timer = self:spawnScenario()
		end
	end
end

-- Input events.
function LuaMainGame:onInputEvents( game, events )
	-- If we're paused, exit.
	if self.owner.paused or self.ninja.lua.dead or self.ninja.lua.bonus then
		return
	end
	
	-- Loop through the events.
	for event in events:all() do
		local touch = TouchEvent.fast_cast( event )
                
        -- If we tapped, jump.
        if touch and touch.type == TouchEvent.TouchStart then
			self.ninja.lua:jump()
			
            events:erase( event )
			
			break
		end        
	end
end

-- Intro sequence.
function LuaMainGame:intro( replay )
	local game = self.owner
	--Audio.get():restoreState()
	
	-- Start paused.
	game.paused = true
	
	-- Spawn a fade in menu.
	do
		local callback = function( inner )
			if( not UserData.get():getFreeShieldsIssued() ) then
				
				self:onNewIAP()
				
			else
				game.paused = false
			end
			
		end
		
		if replay then
			chunks.fade.fade_in( self.hud, callback )
		else
			chunks.fade.fade_in_loading( self.hud, callback )
		end
	end
	
	start_boost_in_progress = false
end

-- The game over event.
function LuaMainGame:onGameOver(first_time)
	local game = self.owner
	self.game_over = true
	game.paused = true
	
	-- Run game over visitor.
	do
		local callback = function( inner )
			if inner.lua and inner.lua.onGameOver then
				inner.lua:onGameOver()
			end
		end
		
		chunks.visitors.visit( game:getScene(), callback )
	end

    first_time = ( ( first_time == nil ) and true or first_time )
	
	-- Spawn the death menu.
	self.death_menu = self.hud:spawn( chunks.death_menu.spawn() )
    self.death_menu.lua.m_play_death_sound = first_time
    self.death_menu.lua:show()
    
    if self.hud.lua.m_shield_button and self.hud.lua.m_pause_button then
    
        -- Disable functionality
        self.hud.lua.m_shield_button.on_enter = function() end
        self.hud.lua.m_shield_button.on_exit = function() end
        self.hud.lua.m_pause_button.on_enter = function() end
        self.hud.lua.m_pause_button.on_exit = function() end

        local s = SequenceAction()	
        s:addAction( ColorAction( 0.4, InterpAction.Smooth, colorf(0.6, 0.6, 0.6, 1.0) ) )
        self.hud.lua.m_shield_button:addAction(s)
        
        s = SequenceAction()	
        s:addAction( ColorAction( 0.4, InterpAction.Smooth, colorf(0.6, 0.6, 0.6, 1.0) ) )
        self.hud.lua.m_pause_button:addAction(s)
    end
    
	-- Update score.
	UserData.get():setHighScore( engine.theme, self.height )
	UserData.get():flush()
	
	print("LUA GAME OVER")
	
	events.postScoreToLeaderboard(game.lua.height)
	events.showRatingsDialog(game.lua.height)
	events.showNotificationsConfirmationPrompt()
	
	events.on_game_over(game.lua.height)
end

-- Show the new IAP information menu
function LuaMainGame:onNewIAP()
	local game = self.owner
	game.paused = true
	
	-- Run game over visitor.
	do
		local callback = function( inner )
			if inner.lua and inner.lua.onGameOver then
				inner.lua:onNewIAP()
			end
		end
		
		chunks.visitors.visit( game:getScene(), callback )
	end
	
	-- Spawn the IAP info menu.
	self.hud.lua:getSpawningGround():spawn( chunks.shield_info.spawn() )

end

function LuaMainGame:ignoreNextPauseEvent()
    --this is called when going out to watch a fuse ad, so that the game doesn't pause upon your return.
    print("*** ignore next pause")
    ignore_next_pause_event = true
end

function LuaMainGame:ceaseIgnoringPauseEvents()
    print("*** dont ignore next pause")
    ignore_next_pause_event = false
end

-- The pause event.
function LuaMainGame:onPause()
    print("*** Pause event recieved")
    if(ignore_next_pause_event)then
        ignore_next_pause_event = false
        print("*** But ignored")
        return
    end
    
	local game = self.owner
	
	if game.paused == false and self.ninja.lua.dead == false then
		game.paused = true
		self.hud.lua.meter.lua:pause()
		self.pause_menu = self.hud:spawn( chunks.pause_menu.spawn() )
		
		-- Run pause visitor.
		do
			local callback = function( inner )
				if inner.lua and inner.lua.onPause then
					inner.lua:onPause()
				end
			end
			
			chunks.visitors.visit( game:getScene(), callback )
		end
	end
end

-- The unpause event.
function LuaMainGame:onUnpause()
	local game = self.owner
	
	if game.paused then
		game.paused = false
		self.hud.lua.meter.lua:unPause()
		
		-- Run unpause visitor.
		do
			local callback = function( inner )
				if inner.lua and inner.lua.onUnpause then
					inner.lua:onUnpause()
				end
			end
			
			chunks.visitors.visit( game:getScene(), callback )
		end
	end
end

-- The quit event.
function LuaMainGame:onQuit()
	local game = self.owner 
	
	-- Spawn a fade out menu.
	do
		local callback = function( inner )
			-- Analytics.
			events.on_quit_game()
			
			game:destroy()
			engine:spawnMenuGame()
		end
		
		chunks.fade.fade_out( self.hud, callback )
	end
end

-- The replay event.
function LuaMainGame:onReplay()
	local game = self.owner 

	-- Spawn a fade out menu.
	do
		local callback = function( inner )
			game:destroy()
			engine:spawnMainGame( engine.theme, true )
			Audio.get():restoreState()
		end
		
		chunks.fade.fade_out( self.hud , callback )
	end
end

function LuaMainGame:getNinja()
	return self.ninja
end

function LuaMainGame:getHud()
	return self.hud
end

function LuaMainGame:getPauseMenu()
	return self.pause_menu
end

function LuaMainGame:getBackground()
	return self.background
end

-- Force the game to pause without bringing up the pause menu
function LuaMainGame:forcePause()
	local game = self.owner
	
	print("FORCE_PAUSE")
	
	if game.paused == false and self.ninja.lua.dead == false then
		game.paused = true
		self.hud.lua.meter.lua:pause()
	end

    self:silenceFootsteps()
end

-- Force the game to unpause without bringing up the pause menu
function LuaMainGame:forceUnpause()
	local game = self.owner
	game.paused = false

    self:unsilenceFootsteps()
end

function LuaMainGame:silenceFootsteps()
    local fs = self.ninja.lua.footsteps_sound

    if fs then
        fs.volume = 0.0
    end
end

function LuaMainGame:unsilenceFootsteps()
    local fs = self.ninja.lua.footsteps_sound

    if fs then
        fs.volume = 1.0
    end
end

function LuaMainGame:isGameOver()
	return self.game_over
end






