------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Gargoyle           --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local assets    = chunks.assets
local pool      = chunks.pool
local enemy     = chunks.enemy
local particles = chunks.particles
local actions   = chunks.actions
local utility   = chunks.utility
local textures  = chunks.textures
local sound     = chunks.sound

-- LuaGargoyleFrozen --
LuaGargoyleFrozen = chunks.class.inherit( chunks.enemy.LuaEnemy )
{
	offset = -102.5
}	

-- The spawner.
spawn_frozen = chunks.class.spawner( ActorBase, LuaGargoyleFrozen )

-- The created method.
function LuaGargoyleFrozen:created( owner )
	-- Initialize super.
	enemy.LuaEnemy.created( self, owner )
	
	-- Initialize owner.
	owner.label    = "gargoyle_frozen"
	owner.location = pool.get_v3f( self.offset, 0.0, 0.0 )
	
	-- Thrower.
	do
		local actor = owner:spawn( SpriteActor() )
		
		textures.setTexture( actor, assets.textures.gargoyle_frozen )

		self.visuals.gargoyle_frozen = actor
	end
	
	self:setVisual( "gargoyle_frozen" )
	
	-- We're not killable.
	self.killable = false
end

-- Setup the easy sequence.
function LuaGargoyleFrozen:easy( callback )
	local actor = self.owner
	
	-- Actions
	do
		local s     = SequenceAction()
		local awake = function( inner )
			-- Spawn particles.
			do
				particles.spawn_big( self.owner:getGame().lua.root, 
									 assets.textures.particles_smoke, 
									 self.visual:getWorldLocation() )
			end
			
			-- Fire the callback.
			callback( self.visual )
			
			-- Play sound.
			sound.play_random( assets.sounds.hit_overhang )
			
			-- Actions.
			do
				local s = SequenceAction()
				
				s:addAction( ScaleAction( 0.1, InterpAction.Smooth, pool.get_v3f( 0.0, 0.0, 1.0 ) ) )
				
				self.visual:addAction( s )
			end
		end
		
		s:addAction( SleepAction( 0.6 ) )
		s:addAction( LuaAction( 0.0, awake ) )
		
		actor:addAction( s )
	end
end

-- LuaGargoyle --
LuaGargoyle = chunks.class.inherit( chunks.enemy.LuaEnemy )()

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaGargoyle )

-- The created method.
function LuaGargoyle:created( owner )
	-- Initialize super.
	enemy.LuaEnemy.created( self, owner )
	
	-- Initialize owner.
	owner.label = "gargoyle"
end

-- Bonus meter override.
function LuaGargoyle:bonusmeterOverride( actor )
    actor.scale = pool.get_v3f( 0.6, 0.6, 1.0 )
    
    textures.setTexture( actor, assets.textures.gargoyle )
end

-- The collided event.
function LuaGargoyle:onCollided( other, is_bonus )
	local game    = self.owner:getGame()
	local ninja   = game.lua.ninja
	local spawner = is_bonus and particles.spawn_uber or particles.spawn_normal
	
	-- Spawn particles.
	do
		spawner( self.owner:getGame().lua.root, 
				 assets.textures.particles_smoke, 
				 self.visual:getWorldLocation() )
	end
	
	self.owner:clearActions()
	actions.shrink_and_kill( self.visual )
	
	-- Play sound.
	sound.play_random( assets.sounds.hit_gargoyle )
end

-- Setup easy sequence.
function LuaGargoyle:easy( instigator )
	local owner  = self.owner
	local vdist1 = -340
	local vdist2 = 160
	local hdist  = 320.0
	local delay  = 3.5
	
	-- Spawn the gargoyle.
	do
		local actor = owner:spawn( CollidableActor() )
		
		textures.setTexture( actor, assets.textures.gargoyle )

		self.visuals.gargoyle = actor
	end
	
	self:setVisual( "gargoyle" )
	
	-- Snap to the frozen gargoyle's location.
	local pos = instigator:getWorldLocation()
	
	if pos.x > 0.0 then
		pos.x = pos.x * -1
	end
	
	owner.location = pos
	owner.scale    = pool.get_v3f( 0.0, 0.0, 1.0 )
	
	-- Play sound.
	sound.play_random( assets.sounds.gargoyle_awake )
	
	-- Actions (owner).
	do
		local split = SplitAction()
		local s1    = SequenceAction()
		local s2    = SequenceAction()
		local s3    = SequenceAction()
		
		s1:addAction( MoveOffsetAction( delay / 2.0, InterpAction.Lerp, pool.get_v3f( hdist * 0.2, 0.0, 0.0 ) ) )
		s1:addAction( MoveOffsetAction( delay / 2.0, InterpAction.Lerp, pool.get_v3f( hdist * 0.8, 0.0, 0.0 ) ) )
		s2:addAction( MoveOffsetAction( delay / 2.0, InterpAction.Smooth, pool.get_v3f( 0.0, vdist1, 0.0 ) ) )
		s2:addAction( MoveOffsetAction( delay / 2.0, InterpAction.Smooth, pool.get_v3f( 0.0, vdist2, 0.0 ) ) )
		s3:addAction( ScaleAction( 0.4, InterpAction.Smooth, pool.get_v3f( 1.0, 1.0, 1.0 ) ) )
		
		split:addAction( s1 )
		split:addAction( s2 )
		split:addAction( s3 )
		
		owner:addAction( split )
	end  
end

-- Setup hard sequence.
function LuaGargoyle:hard( instigator )
	local owner  = self.owner
	local vdist1 = -340
	local vdist2 = 160
	local hdist  = 320.0
	local delay  = 2.5
	
	-- Spawn the gargoyle.
	do
		local actor = owner:spawn( CollidableActor() )
		
		textures.setTexture( actor, assets.textures.gargoyle )

		self.visuals.gargoyle = actor
	end
	
	self:setVisual( "gargoyle" )
	
	-- Snap to the frozen gargoyle's location.
	local pos = instigator:getWorldLocation()
	
	if pos.x > 0.0 then
		pos.x = pos.x * -1
	end
	
	owner.location = pos
	owner.scale    = pool.get_v3f( 0.0, 0.0, 1.0 )
	
	-- Play sound.
	sound.play_random( assets.sounds.gargoyle_awake )
	
	-- Actions (owner).
	do
		local split = SplitAction()
		local s1    = SequenceAction()
		local s2    = SequenceAction()
		local s3    = SequenceAction()
		
		s1:addAction( MoveOffsetAction( delay / 2.0, InterpAction.Lerp, pool.get_v3f( hdist * 0.2, 0.0, 0.0 ) ) )
		s1:addAction( MoveOffsetAction( delay / 2.0, InterpAction.Lerp, pool.get_v3f( hdist * 0.8, 0.0, 0.0 ) ) )
		s2:addAction( MoveOffsetAction( delay / 2.0, InterpAction.Smooth, pool.get_v3f( 0.0, vdist1, 0.0 ) ) )
		s2:addAction( MoveOffsetAction( delay / 2.0, InterpAction.Smooth, pool.get_v3f( 0.0, vdist2, 0.0 ) ) )
		s3:addAction( ScaleAction( 0.4, InterpAction.Smooth, pool.get_v3f( 1.0, 1.0, 1.0 ) ) )
		
		split:addAction( s1 )
		split:addAction( s2 )
		split:addAction( s3 )
		
		owner:addAction( split )
	end  
end

-- The bonus setup method.
function LuaGargoyle:bonus( ninja )
	local owner   = self.owner
	local offset  = ninja.lua.sides.left.offset
	local dir     = ( ninja:getWorldLocation().x < 0.0 ) and 1 or -1
	local xdist   = 200.0
	local zdist   = -100.0
	local ydist   = 320.0
	local bend    = v3f( math.abs( offset.x ) * dir, offset.y, offset.z )
	local bstart  = v3f( ( math.abs( offset.x ) + 140.0 ) * dir, bend.y, offset.z + zdist )
	local bscale  = 0.5
	local fscale  = 1.5
	local delay   = 7.0
	local set_rot = function( inner, d )
		local r = v3f( 0.0, ( d == -1 ) and 180.0 or 0.0, 0.0 )
		
		ninja.lua:setRotation( r, v3f( 0.0, r.y - 180.0, 0.0 ) )
	end
	
	-- Set visual.
	textures.setTexture( ninja.lua.visual, assets.textures.bonus_gargoyle )
	
	-- Enlarge shield.
	ninja.lua.shield.lua:bonusStart( 2.4 )
	
	-- Actions.
	do
		local s       = SequenceAction()
		local spirals = 4
		
		-- Initial scaling / z movement.
		do
			local split = SplitAction()
			local s1    = SequenceAction()
			local s2    = SequenceAction()
			local pcent = 1.0 / ( spirals * 2 + 2 )
			
			-- Rotation.
			s:addAction( LuaAction( 0.0, set_rot, dir ) )
			
			-- Scaling.
			s1:addAction( ScaleAction( delay * pcent * 0.5, InterpAction.Smooth,
			              pool.get_v3f( bscale, bscale, 1.0 ) ) )
			s1:addAction( ScaleAction( delay * pcent * 0.5, InterpAction.Smooth,
			              pool.get_v3f( 1.0, 1.0, 1.0 ) ) )
			              
			-- Movement.   
			s2:addAction( MoveOffsetAction( 0.0, InterpAction.Smooth,
			              pool.get_v3f( 0.0, 0.0, zdist ) ) )
			s2:addAction( MoveAction( delay * pcent, InterpAction.Smooth, bstart ) )
			
			split:addAction( s1 )
			split:addAction( s2 )
			s:addAction( split )		
		end
		
		-- Spiral movement.
		do
			local split = SplitAction()
			local s1    = SequenceAction()
			local s2    = SequenceAction()
			local s3    = SequenceAction()
			local pcent = ( spirals * 2 ) / ( spirals * 2 + 2 )
						
			-- Vertical movement.
			s1:addAction( MoveOffsetAction( delay * pcent * 0.5, InterpAction.Smooth,
			              pool.get_v3f( 0.0, ydist, 0.0 ) ) )
			s1:addAction( MoveOffsetAction( delay * pcent * 0.5, InterpAction.Smooth,
			              pool.get_v3f( 0.0, -ydist, 0.0 ) ) )
			              
			-- Spirals.
			do
				local spcent = pcent * ( 1 / spirals )
				
				for i = 1,spirals do	
					-- Horizontal movement.
					s2:addAction( LuaAction( 0.0, set_rot, dir * -1 ) )
					s2:addAction( MoveOffsetAction( 0.0, InterpAction.Smooth,
					              pool.get_v3f( 0.0, 0.0, -zdist ) ) )
					s2:addAction( MoveOffsetAction( delay * spcent * 0.5, InterpAction.Smooth,
					              pool.get_v3f( xdist * dir * -1, 0.0, 0.0 ) ) )
					s2:addAction( LuaAction( 0.0, set_rot, dir ) )
					s2:addAction( MoveOffsetAction( 0.0, InterpAction.Smooth,
					              pool.get_v3f( 0.0, 0.0, zdist ) ) )
					s2:addAction( MoveOffsetAction( delay * spcent * 0.5, InterpAction.Smooth,
					              pool.get_v3f( xdist * dir, 0.0, 0.0 ) ) )
					              
					-- Scaling.
					s3:addAction( ScaleAction( delay * spcent * 0.25, InterpAction.Smooth,
					              pool.get_v3f( fscale, fscale, 1.0 ) ) )
					s3:addAction( ScaleAction( delay * spcent * 0.25, InterpAction.Smooth,
					              pool.get_v3f( 1.0, 1.0, 1.0 ) ) )
					s3:addAction( ScaleAction( delay * spcent * 0.25, InterpAction.Smooth,
					              pool.get_v3f( bscale, bscale, 1.0 ) ) )
					s3:addAction( ScaleAction( delay * spcent * 0.25, InterpAction.Smooth,
					              pool.get_v3f( 1.0, 1.0, 1.0 ) ) )
				end
			end
			
			split:addAction( s1 )
			split:addAction( s2 )
			split:addAction( s3 )
			s:addAction( split )	
		end
		
		-- Move back to starting position.
		do
			local split = SplitAction()
			local s1    = SequenceAction()
			local s2    = SequenceAction()
			local pcent = 1.0 / ( spirals * 2 + 2 )
			
			-- Rotation.
			s:addAction( LuaAction( 0.0, set_rot, dir * -1 ) )
			
			-- Scaling.
			s1:addAction( ScaleAction( delay * pcent * 0.5, InterpAction.Smooth,
			              pool.get_v3f( fscale, fscale, 1.0 ) ) )
			s1:addAction( ScaleAction( delay * pcent * 0.5, InterpAction.Smooth,
			              pool.get_v3f( 1.0, 1.0, 1.0 ) ) )
			              
			-- Movement.   
			s2:addAction( MoveOffsetAction( 0.0, InterpAction.Smooth,
			              pool.get_v3f( 0.0, 0.0, -zdist ) ) )
			              
			              print( pcent )
			              
			s2:addAction( MoveAction( delay * pcent, InterpAction.Smooth, bend ) )
			
			split:addAction( s1 )
			split:addAction( s2 )
			s:addAction( split )		
		end

		-- Done.		
		s:addAction( LuaMemberAction( 0.0, "bonusStop" ) )	
		
		ninja:addAction( s )
	end	
	
	-- Play bonus sound.
	sound.play( assets.sounds.bonus_gargoyle )
	
	return delay
end

-- Update the ninja's trailer.
function LuaGargoyle:updateTrailer( trailer )
	local s = SequenceAction()
			
	s:addAction( ColorAction( 0.4, InterpAction.Smooth, 
	             pool.get_colorf( 54.0 / 255.0, 46.0 / 255.0, 59.0 / 255.0, 1.0 ) ) )
	
	trailer:addAction( s )	
end