--| module |--
module(...,package.seeall)

PRODUCT_DISPLAY_ORDER = {"shield_tier1", "shield_tier2", "shield_tier3", "shield_tier4", "shield_tier5"}
PRODUCTS = {
    shield_tier1  = { value = 2,    description = "2"   },
    shield_tier2  = { value = 5,    description = "5"   },
    shield_tier3  = { value = 15,   description = "15"  },
    shield_tier4  = { value = 50,   description = "50"  },
    shield_tier5  = { value = 200,  description = "200" },
    ad_buyout = { description = "Ad Buyout" }
}

local ui_handle = nil
local request_handle = nil

function purchasingAvailable()
    local iap = InAppPurchaseService.get()
    return (iap ~=nil and iap:canPurchase())
end

function displayPurchasingUnavailable(callback)
    local ui = PlatformUIService.get()

    if ui_handle then
        --Cancel any existing calls (which should never exist)
        ui_handle:cancel()
        ui_handle = nil
    end

    local cb = function()
        ui_handle = nil
        if callback then
            callback()
        end
    end

    if ui then
        local title = lstring("purchasing_unavailable_title")
        local text = lstring("purchasing_unavailable_text")
        local ok = lstring("ok")

        ui_handle = ui:showDialog(title, text, ok, cb)
    end
end


function adBuyoutPurchased()
    local ad_buyout_purchased = false
    local iap = InAppPurchaseService.get()
    if iap then
        local num_buyout = iap:numberPurchasedForProduct("ad_buyout")
        if num_buyout and num_buyout > 0 then
            ad_buyout_purchased = true

            -- FIX FOR ANDROID, if buyout has been purchased then force ads to be disabled.
            local ads = AdsService.get()
            if ads ~= nil then
                ads:disableAds()
            end
        end
    end

    return ad_buyout_purchased
end

function getPriceString(id)
    local iap = InAppPurchaseService.get()
    
    if iap then
        local product = iap:getProduct(id)
        print("Product for id ", id, " is ", product)
        if product then
            return product:getPriceString()
        end
    end

    return ""
end

function purchaseProduct(product_id, response_complete_callback, products_consumed_callback)
    local request_handle = nil

    -- Make sure the product exists
    local product_details = PRODUCTS[product_id]
    if not product_details then
        print("iAP.lua: Could not find a product with id: ", product_id)
        return request_handle
    end
    local iap = InAppPurchaseService.get()
    if iap then
        local callback = function(handle, code, result)
            print("iAP.lua: Trying to purchase product id: " .. tostring(product_id))
            local consumed_list = iap:consumePurchases(product_id);

            --NOTE: This could include previously purchased products even if the current purchase was a failure.
            local num_consumed = #consumed_list
            print("iAP.lua: Num Consumed: " .. tostring(num_consumed))

            if num_consumed > 0 then
                awardProduct(product_id, num_consumed)

                if products_consumed_callback then
                    products_consumed_callback(product_id, num_consumed)
                end
            end

            if code == IAPRequestResponseCode.ResponseCode_Success then
                --Notify the user
                -- This isn't necessary as there is visual feedback in shieldstore.lua (or in the hud)
            elseif code ~= IAPRequestResponseCode.ResponseCode_UserCancelled then
                local platform_ui = PlatformUIService.get();
                if platform_ui then
                    local title = lstring("unsuccessful_purchase_title")
                    local text = lstring("unsuccessful_purchase_text")
                    local ok = lstring("ok")
                    platform_ui:showDialog(title, text, ok, function() end)
                end
            end

            if response_complete_callback then
                --This code executes once the response has been received (used for clearing any handles, etc)
                response_complete_callback()
            end
        end -- end of callback

        request_handle = iap:requestPurchaseProduct(product_id, callback)
    end
    
    return request_handle
end

function awardProduct(product_id, num_consumed)

    --putting these refs here instead of at file level because of a wierd include thing we are doing with this file at startup
    local engine   = engine.instance
    local chunks   = engine.chunks

    local product_details = PRODUCTS[product_id]

    if product_id == "ad_buyout" then
        local ads = AdsService.get()

        if ads ~= nil then
            ads:disableAds()
        end
    else
        
        -- Add Shield Points
        local user_data = UserData.instance()
        local amount = num_consumed * product_details.value
        
        local capacity = user_data:getShieldCapacity()
        user_data:setShieldCapacity(capacity + amount)
        
        local shields = user_data:getShields()
        user_data:setShields(shields + amount)
        chunks.analytics.reportShieldPurchase(shields, amount)
    end
end


function restorePurchases(request_complete_callback)
    local handle = nil

    local iap = InAppPurchaseService.get();
    if iap then
        local callback = function(handle, status, response)
            local count = 0
            print("Restore result: " .. tostring(status))
            if status == 0 then
                local raw_product_list = iap:consumeOutstandingPurchases();
                local products_restored = {}

                for _,entry in ipairs(raw_product_list) do
                    local num = products_restored[entry.productName] and products_restored[entry.productName] or 0
                    products_restored[entry.productName] = num + 1
                end

                for name, number in pairs(products_restored) do
                    if name == "ad_buyout" then
                        count = count + number

                        awardProduct(name, number)
                        print("Product: " .. name .. " (" .. tostring(number) .. ")")
                    else                				
                        print("Products Restored")

                        local product = PRODUCTS[name]
                        local value = 0

                        if product then
                            points = product.value * number
                            awardProduct(name, number)

                            count = count + number
                            print("Restoring " .. points .. " shields")
                        else
                            print("Unable to restore unknown product: " .. name)
                        end
                    end
                end
            end

            print("Total products restored: " .. tostring(count))

            local ui = PlatformUIService.get()
            if ui then
                local title = lstring("restore_purchases_result_title")
                local button = lstring("ok")
                local message = nil

                if count > 0 then
                    message = lstring("restore_purchases_result_message")
                    message:arg(1, count)
                    message:arg(2, iap:getRestoreKey())
                else
                    message = lstring("restore_purchases_result_no_products_restored")
                    message:arg(1, iap:getRestoreKey())
                end

                ui:showDialog(title, message, button, function() self:dismiss() end)
            end

            -- Execute request complete callback.
            if request_complete_callback then 
                request_complete_callback()
            end
        end
        
        handle = iap:requestRestorePurchases(callback);
    end

    return handle
end

