------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Ninja              --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine   = engine.instance
local chunks   = engine.chunks
local assets   = chunks.assets
local pool     = chunks.pool
local pawn     = chunks.pawn
local actions  = chunks.actions
local textures = chunks.textures
local sound    = chunks.sound

-- LuaNinja --
LuaNinja = chunks.class.inherit( pawn.LuaPawn ) 
{
	-- side --
	side  = "left",
	sides = 
	{
		left = 
		{
			offset   = v3f( -118.0, -120.0, 10.0 ),
			rotation = v3f( 0.0, 0.0, 0.0 )
		},
		right =
		{
			offset   = v3f( 118.0, -120.0, 10.0 ),
			rotation = v3f( 0.0, 180.0, 0.0 )
		}
	},
	
	-- visuals --
	visual_root = nil,
	
	-- jumping --
	jumping     = false,
	jump_time   = 0.375,
	jump_height = 20.0,
	
	-- enemies --
	enemies = nil,
	
	-- bonus --
	bonus        = false,
	bonus_enemy  = "",
	bonus_count  = 0,
	bonus_factor = 2.0,
	
	-- shield --
	shield = nil,
	
	-- trailer --
	trailer = nil,
	
	-- footsteps --
	footsteps_sound = nil,
	
	-- misc --
	dead = false,
	used_start_boost = false,
	revive_used = false
}

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaNinja )

-- The created method.
function LuaNinja:created( owner )
	-- Initialize super.
	pawn.LuaPawn.created( self, owner )
	
	-- Initialize owner.
	owner.label = "ninja"
	
	-- Visual root.
	self.visual_root = owner:spawn( ActorBase() )
	
	-- Climbing.
	do
		local actor = self.visual_root:spawn( CollidableActor() )
		
		textures.setTexture( actor, assets.textures.ninja_climbing )

		self.visuals.climbing = actor
	end
	
	-- Jumping.
	do
		local actor = self.visual_root:spawn( CollidableActor() )
		
		textures.setTexture( actor, assets.textures.ninja_jumping )
		
		self.visuals.jumping = actor
	end
	
	-- Falling.
	do
		local actor = self.visual_root:spawn( CollidableActor() )
		
		textures.setTexture( actor, assets.textures.ninja_falling )
		
		self.visuals.falling = actor
	end
	
	-- Bonus.
	do
		local actor = self.visual_root:spawn( CollidableActor() )
		
		self.visuals.bonus = actor
	end
	
	-- Shield.
	do
		self.shield = owner:spawn( chunks.shield.spawn() )
	end
	
	-- Trailer.
	do
		local trailer = owner:spawn( ParticleSystem() )
		
		-- constant settings
		trailer.location               = pool.get_v3f( 0.0, -7.0, -1.0 )
		trailer.shape                  = ParticleSystem.Cone
		trailer.texture                = assets.textures.particles_spark.path
		trailer.particle_size          = assets.textures.particles_spark.size
		trailer.lifetime_range         = pool.get_v2f( 0.4, 0.6 )
		trailer.spawn_frequency_range  = pool.get_v2f( 0.04, 0.06 )
		trailer.speed_range            = pool.get_v2f( 150.0, 200.0 )
		trailer.position_range         = pool.get_v2f( 0.0, 0.0 )
		trailer.initial_rotation_range = pool.get_v2f( 0.0, 360.0 )
		trailer.rotation_speed_range   = pool.get_v2f( 80.0, 100.0 )

		-- scale transitions
		trailer:addScaleRange( pool.get_v2f( 0.0, 0.0 ) )
		trailer:addScaleRange( pool.get_v2f( 0.8, 1.2 ) )
		trailer:addScaleRange( pool.get_v2f( 0.8, 1.4 ) )
		trailer:addScaleRange( pool.get_v2f( 0.8, 1.2 ) )
		trailer:addScaleRange( pool.get_v2f( 0.0, 0.0 ) )
		
        trailer:setReferenceActor( owner:getGame() )
        trailer.follow_system = false

		self.trailer = trailer
	end
	
	-- Setup footsteps sound.
	self.footsteps_sound = sound.play( assets.sounds.footsteps )
	
	-- Create enemies table.
	self.enemies = {}
	
	-- Set visual.
	self:setVisual( "climbing" )
	
	-- Set side.
	self:setSide( self.side )
	
	-- Setup initial footsteps delay.
	do
		local start_footsteps = function( inner )
            if self.footsteps_sound then
                self.footsteps_sound.volume = 1.0
            end
		end
		
		-- Actions.
		do
			local s = SequenceAction()
			
			s:addAction( LuaAction( 0.0001, start_footsteps ) )
			
			owner:addAction( s )
		end
		
		-- Mute it initially.
        if self.footsteps_sound then
            self.footsteps_sound.volume = 0.0
        end
	end
end

-- The destroy event.
function LuaNinja:destroy( owner )
	-- Stop footsteps sound.
	if self.footsteps_sound then
		self.footsteps_sound:stop()
		self.footsteps_sound = nil
	end
end

-- On pause.
function LuaNinja:onPause()
	if self.footsteps_sound then
		self.footsteps_sound.volume = 0.0
	end
end

-- On onUnpause.
function LuaNinja:onUnpause()
	if not self.jumping and not self.dead and not self.bonus and self.footsteps_sound then
		self.footsteps_sound.volume = 1.0
	end
end

-- Are we protected?
function LuaNinja:isProtected()
	return self.shield.lua.equipped or self.bonus or self.dead 
end

-- The tick event.
function LuaNinja:tick( owner, delta_time )
	local game = owner:getGame()
			
	-- Check for collisions.
	for k,v in pairs( self.enemies ) do
		if v ~= nil and v.lua:collides( owner ) then
			-- Unregister the enemy.
			v.lua:unregister()
			
			local is_shieldpack = ( v.label == "shieldpack" )
			local is_killable   = ( self.jumping and v.lua.killable )
			
			-- Killable?
			if is_killable or is_shieldpack or self:isProtected() then
				-- Grabbed shield?
				if is_shieldpack then
					self.shield.lua:equip()
				-- Lost shield?
				elseif not is_killable and self.shield.lua.equipped and not self.bonus then
					self.shield.lua:lose()
					self:pulsateShieldButton()
				end
				
				-- Check bonus.
				local is_bonus = self:checkBonus( v )
				
				-- Update bonus meter.
				self:updateBonusMeter( v, is_bonus )
				
				-- Collided.
				v.lua:onCollided( owner, is_bonus )
			-- Otherwise, die.
			else
				self:die()
			end
			
			-- Always update the trailer.
			self:updateTrailer( v )
		end
	end
end

-- Set visual rotation.
function LuaNinja:setRotation( visual_rot, shield_rot )
	self.visual_root.rotation = visual_rot
	self.shield.rotation      = shield_rot
end

-- Set side.
function LuaNinja:setSide( side )
	self.side           = side
	self.owner.location = self.sides[ side ].offset
	
	-- Rotation.
	do
		local rot = self.sides[ side ].rotation
	
		self:setRotation( rot, rot )
	end
	
	-- Unmute footsteps sound.
    if self.footsteps_sound then
        self.footsteps_sound.volume = 1.0
    end
end

-- Jump.
function LuaNinja:jump()
	-- If we're already jumping, exit.
	if self.jumping then
		return
	end
	
	-- Set state.
	self.jumping = true
	
	-- Figure out target side.
	local target = "right"
	
	if self.side == "right" then
		target = "left"
	end
	
	-- Set visual.
	self:setVisual( "jumping" )
	
	-- Update shield rotation.
	do
		local rot = self.shield.rotation
		
		self.shield.rotation = pool.get_v3f( rot.x, rot.y + 180.0, rot.z )
	end
	
	-- Mute footsteps sound.
    if self.footsteps_sound then
        self.footsteps_sound.volume = 0.0
    end
	
	-- Play sound.
	sound.play_random( assets.sounds.ninja_jump )
	
	-- Setup the jump.
	do
		local s = SequenceAction()
		
		-- Movement.
		do
			local s2 = SplitAction()
		
			-- Horizontal movement.
			s2:addAction( MoveAction( self.jump_time, InterpAction.Lerp, self.sides[ target ].offset ) )
			
			-- Vertical movement.
			do
				local s3 = SequenceAction()
				
				s3:addAction( MoveOffsetAction( self.jump_time / 2.0, InterpAction.Smooth, 
				                                pool.get_v3f( 0.0,  self.jump_height, 0.0 ) ) )
				s3:addAction( MoveOffsetAction( self.jump_time / 2.0, InterpAction.Smooth, 
				                                pool.get_v3f( 0.0, -self.jump_height, 0.0 ) ) )
				
				s2:addAction( s3 )
			end
			
			s:addAction( s2 )
		end
		
		-- Finishing.
		do
			local f = function( inner )
				self:setSide( target )
				self:setVisual( "climbing" )
				
				-- Play sound.
				sound.play_random( assets.sounds.ninja_land )
			end
			
			s:addAction( LuaAction( 0.0, f ) )
			
			-- Wait a little longer to disable jumping.
			s:addAction( LuaAction( 0.025, function( inner ) self.jumping = false end ) )
		end	
		
		self.owner:clearActions()
		self.owner:addAction( s )
	end
end

-- Die.
function LuaNinja:die()
	local game  = self.owner:getGame()
	local owner = self.owner
	
	local fall_time = 2.5
	
	
	-- Start death sequence.
	do
		-- Set state.
		self.dead = true
		
		-- Set visual.
		self:setVisual( "falling" )
		
		-- Update rotation.
		do
			local rot = v3f( 0.0, 180.0, 0.0 )
			
			if owner:getWorldLocation().x < 0.0 then
				rot = v3f( 0.0, 0.0, 0.0 )
			end
			
			self:setRotation( rot, rot )
		end
		
		-- Mute footsteps sound.
        if self.footsteps_sound then
            self.footsteps_sound.volume = 0.0
		end

		-- Stop trailer particles.
		self.trailer.particles_per_spawn_range = pool.get_v2i( 0, 0 )
		
		-- Clear bonus meter.
		game.lua.hud.lua.meter.lua:clear()
		
	end

	-- Setup ending sequence.
	do
		local walls         = game.lua.walls
		local root          = game.lua.scenario_root
		local s             = SequenceAction()
		local reverse_walls = function( inner )
			-- Reverse the walls.
			walls.speed = game.lua.speed * -1
			
			-- Stop actions on all scenarios.
			do
				local function recurser( parent )
					for i = 0,parent:getNumOwned() do
						local child = parent:getOwned( i )
						
						if child then
							if child.label == "scenario" then 
								child:clearActions()
							end
							
							recurser( child )
						end
					end
				end
				
				recurser( root )
			end
			
			-- Stop backgrounds.
			game.lua.background.lua:stop()
			
			-- Move root upwards.
			do
				local s2       = SequenceAction()
				local delay    = fall_time
				local distance = math.abs( game.lua.speed * delay )
				
				s2:addAction( MoveOffsetAction( delay, InterpAction.Lerp, 
				                                pool.get_v3f( 0.0, distance, 0.0 ) ) )
				root:clearActions()
				root:addAction( s2 )
			end
			
			-- Play falling sound.
			sound.play_random( assets.sounds.ninja_fall )
		end
		local stop_walls = function( inner )
			walls.speed = 0.0
			root:clearActions()
		end	
		
		
		s:addAction( LuaAction( 0.0, reverse_walls ) )
		s:addAction( MoveAction( 0.5, InterpAction.Smooth, pool.get_v3f( 0.0, -130.0, owner.location.z ) ) )
		s:addAction( SleepAction( fall_time - 1.0 ) )
		s:addAction( LuaAction( 0.0, stop_walls ) )
		s:addAction( MoveAction( 0.5, InterpAction.Smooth, pool.get_v3f( 0.0, -330.0, owner.location.z ) ) )
		
	
		self.owner:clearActions()
		self.owner:addAction( s )
	end

	local game_over  = function( inner )
	    self.owner:addAction( LuaAction( 0.0, function() game.lua:onGameOver() end ) )
	    
	    --send the analytics death event regardless of whether the user activated the post-roll incent video saving move on some previous almost-death
	    --apdated for new specs--also report whether the start boost was used
	    local start_boost_was_offered = game.lua.hud.lua.m_start_boost_offered
	    local revive_was_offered = game.lua.hud.lua.m_end_boost_offered
		
		if not GameService.get():zeb() then
			chunks.analytics.reportEndRun(self.owner:getGame().lua.height, self.used_start_boost, start_boost_was_offered, self.revive_used, revive_was_offered)
		end
	
	end
		
	game.lua.hud.lua:ninjaDeath(game_over, fall_time - 0.5)
	
	
end

function LuaNinja:startBoostUsed()
    print("Report start boost used")
    self.used_start_boost = true
end

function LuaNinja:reviveUsed()
    print("Report revive used")
    self.revive_used = true
end

-- Check if we need to start a bonus.
function LuaNinja:checkBonus( other )
	-- Exit conditions:
	-- 1) we're dead
	-- 2) we're in a bonus already
	-- 3) the enemy is a shieldpack
	if self.dead or self.bonus or other.label == "shieldpack" then
		return false
	end
	
	-- Update accounting.
	if self.bonus_enemy == other.label then
		self.bonus_count = self.bonus_count + 1
	else
		self.bonus_enemy = other.label
		self.bonus_count = 1
	end
	
	-- Bonus?  Here are the conditions:
	-- 1) Our bonus count is 3.
	-- 2) The enemy has a bonus method.
	if self.bonus_count >= 3 and other.lua.bonus ~= nil then
		self:bonusStart( other )
		
		return true
	end
	
	return false
end

-- Start a bonus.
function LuaNinja:bonusStart( other )
	local game = self.owner:getGame()
	
	-- Set state.
	self.bonus   = true
	self.jumping = false
	
	-- Set visual.
	self:setVisual( "bonus" )
	
	-- Clear actions.
	self.owner:clearActions()
	
	-- Mute footsteps sound.
    if self.footsteps_sound then
        self.footsteps_sound.volume = 0.0
    end

	-- Apply bonus factor.
	do
		game.lua.walls.speed                     = game.lua.speed * self.bonus_factor
		game.lua.scenario_root.delta_time_factor = self.bonus_factor
	end
	
	-- Start bonus.
	do
		local delay = other.lua:bonus( self.owner )
		
		game.lua.bonus_timer = delay + game.lua.bonus_extra_time
	end
end


function LuaNinja:incentiveRevival( callback )

    local speed_factor = 2.0
	local game = self.owner:getGame()
	game.lua:beginStartBoost()
	
	-- Move root back into place
	local root          = game.lua.scenario_root
	do
		local s2       = SequenceAction()
		local delay    = 2.0
		local distance = -math.abs( game.lua.speed * delay )
				
		s2:addAction( MoveAction( delay, InterpAction.Lerp, 
				                                pool.get_v3f( 0.0, 0.0, 0.0 ) ) )
		s2:addAction( LuaAction ( 0.0, function() self.dead = false end ) )
		root:addAction( s2 )
	end
	
	-- Stop actions on all scenarios.
	do
		local function recurser( parent )
			for i = 0,parent:getNumOwned() do
				local child = parent:getOwned( i )
						
				if child then
					if child.label == "scenario" then 
						child:destroy()
					end
							
					recurser( child )
				end
			end
		end
				
		recurser( root )
	end
					
	-- Set state.
	self.bonus   = true
	self.jumping = false
	
	-- Set visual.
	self:setVisual( "bonus" )
	
	-- Clear actions.
	self.owner:clearActions()

    -- Restart backgrounds.
	game.lua.background.lua:restart()
			
	-- Apply bonus factor.
	do
		game.lua.walls.speed                     = game.lua.speed * speed_factor
		game.lua.scenario_root.delta_time_factor = speed_factor
	end
	
	-- Revive rocket
	do
	    local projectile = self.owner:spawn( chunks.start_boost.spawn() )
	    local delay = projectile.lua:startBonus( self.owner, 2.0, callback )
		game.lua.bonus_timer = delay
	end
end


-- Force start a start bonus following an incentivized ad
function LuaNinja:incentiveBonusStart( boost_distance, callback )

    local speed_factor = 3.0
	local game = self.owner:getGame()
	game.lua:beginStartBoost()
	
	-- Set state.
	self.bonus   = true
	self.jumping = false
	
	-- Set visual.
	self:setVisual( "bonus" )
	
	-- Clear actions.
	self.owner:clearActions()
	
	-- Mute footsteps sound.
    if self.footsteps_sound then
        self.footsteps_sound.volume = 0.0
    end

	-- Apply bonus factor.
	do
		game.lua.walls.speed                     = game.lua.speed * speed_factor
		game.lua.scenario_root.delta_time_factor = speed_factor
	end
	
	-- Start bonus.
	do
	    local projectile = self.owner:spawn( chunks.start_boost.spawn() )
	    local boost_time = boost_distance / (200)
	    local delay = projectile.lua:startBonus( self.owner, boost_time, callback )
		game.lua.bonus_timer = delay
	end
end

-- Stop a bonus.
function LuaNinja:bonusStop()
	local game = self.owner:getGame()
	game.lua:endStartBoost();
	
	-- Set state.
	self.bonus       = false
	self.bonus_enemy = ""
	self.bonus_count = 0
	
	-- Set side.
	if self.owner:getWorldLocation().x < 0.0 then
		self:setSide( "left" )
	else
		self:setSide( "right" )
	end
	
	-- Set visual.
	self:setVisual( "climbing" )
	
	-- Shrink shield.
	self.shield.lua:bonusStop()
	
	-- Stop trailer particles.
	self.trailer.particles_per_spawn_range = pool.get_v2i( 0, 0 )

	-- Undo bonus factor.
	do
		game.lua.walls.speed                     = game.lua.speed 
		game.lua.scenario_root.delta_time_factor = 1.0
	end
	
	-- Clear bonus meter.
	game.lua.hud.lua.meter.lua:clear()
	
	-- buffer end scenario
end

-- Update trailer.
function LuaNinja:updateTrailer( other )
	local trailer = self.trailer
	local count   = self.bonus_count
	
	-- Exit but don't turn off particles.  Conditions are:
	-- 1) we're dead
	-- 2) we're in a bonux
	-- 3) other is a shieldpack
	if self.dead or self.bonus or other.label == "shieldpack" then
		return
	end
	
	-- Exit and turn off particles if other 
	-- has no updateTrailer method.
	if other.lua.updateTrailer == nil then
		trailer.particles_per_spawn_range = pool.get_v2i( 0, 0 )
		
		return
	end
	
	-- Update settings.
	trailer.cone_angle_range          = pool.get_v2f( -90.0 - 15.0 * count, -90.0 + 15.0 * count )
	trailer.particles_per_spawn_range = pool.get_v2i( 1 + ( count - 1 ) * 2, 3 + ( count - 1 ) * 2 )
	
	-- Have the actor update the trailer.
	other.lua:updateTrailer( trailer )
end

-- Update the bonus meter.
function LuaNinja:updateBonusMeter( enemy, is_bonus )
	-- Exit conditions:
	-- 1) we're in a bonus
	-- 2) we're dead
	--if not is_bonus and ( self.bonus or self.dead ) then
	if enemy.label == "shieldpack" or self.dead or ( self.bonus and not is_bonus ) then
		return
	end	
	
	local game = self.owner:getGame()
	
	-- Update the meter.
	game.lua.hud.lua.meter.lua:update( enemy, self.bonus_count )
	
	-- Turn on bonus meter animation?
	if is_bonus then
		game.lua.hud.lua.meter.lua:bonus()
	end
end

function LuaNinja:getShield()
	return self.shield
end

function LuaNinja:isDead()
	return self.dead
end

function LuaNinja:pulsateShieldButton()
	self.owner:getGame().lua.hud.lua:pulsateShieldButton()
end

function LuaNinja:activateIAPShield()
	self.owner:getGame().lua.hud.lua:activateShield()
end
