------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- LuaOptionsMenu     --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local constants = engine.platform.constants
local chunks    = engine.chunks
local assets    = chunks.assets
local assets_shared    = chunks.assets_shared
local pool      = chunks.pool
local textures  = chunks.textures
local fade      = chunks.fade
local events    = chunks.events
local utility 	= chunks.utility

-- LuaOptionsMenu --
LuaOptionsMenu = chunks.class.inherit( chunks.window.LuaWindow )
{
	volume_box_offset = v2f( 90.0, 60.0 ),
	score_offset      = v2f( 20.0, -23.0 ),
	button_box_offset = v2f( -60.0, 40.0 ),
    m_iap_handle = nil,
}

-- The spawner.
spawn = chunks.class.spawner( Window, LuaOptionsMenu )

-- The created method.
function LuaOptionsMenu:created( owner )
	local game     = owner:getGame()
	local x_offset = 5.0
	local y_offset = constants.ad_height + 10.0

	-- Background.
	do
		textures.setTexture( owner, assets.textures[ "options_" .. engine.theme ] )
	end

	-- Back.
	do
		local on_release = function( inner ) self:onBack() end
		local actor      = self:addButton( on_release )

		actor.anchor = make_anchor( Anchor.Anchor_BL, Anchor.Anchor_BL, pool.get_v3f( x_offset, y_offset, 1.0 ) )

		textures.setTexture( actor, assets_shared.textures.back_button )

		if not engine.default_language_enabled then
			utility.createText( owner, lstring("options"),
				{"ninja-text-large", 70.0, pool.get_colorf(1, 0, 0, 1), 170, TextAlign.Center, 70, 220} )

			utility.createText( owner, lstring("sound"),
				{"ninja-text", 40.0, pool.get_colorf(0, 0, 0, 1), 105, TextAlign.Center, 25, 141}, { drop_shadow = false } )

			utility.createText( owner, lstring("music"),
				{"ninja-text", 40.0, pool.get_colorf(0, 0, 0, 1), 105, TextAlign.Center, 25, 88}, { drop_shadow = false } )

		else
			local overlay_logo = utility.createWindowWithTexture( self.owner, assets.textures.options_logo_locale )
			overlay_logo.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( 50, 205, 0.005 ) ) --, pool.get_v2f(195, 177) )

			local overlay_sound_text = utility.createWindowWithTexture( self.owner, assets.textures.options_sound_text_locale )
			overlay_sound_text.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( 10, 140, 0.005 ) ) --, pool.get_v2f(195, 177) )

			local overlay_music_text = utility.createWindowWithTexture( self.owner, assets.textures.options_music_text_locale )
			overlay_music_text.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( 10, 85, 0.005 ) ) --, pool.get_v2f(195, 177) )
		end
	end

	local volume_box = nil

	-- Box for volume buttons.
	do
		volume_box         = owner:spawn( chunks.window.spawn( Window() ) )
		volume_box.size    = pool.get_v2f( 100.0, 100.0 )
		volume_box.hidden  = true
		volume_box.anchor  = make_anchor( Anchor.Anchor_BL, Anchor.Anchor_C,
		                             pool.get_v3f( self.volume_box_offset.x, self.volume_box_offset.y, 1.0 ) )
	end

	-- Function to set the volume texture.
	local set_volume_texture = function( actor, on )
		if engine.default_language_enabled then
			if on then
				textures.setTexture( actor, assets.textures.on_button )
			else
				textures.setTexture( actor, assets.textures.off_button )
			end
		else
			actor:destroyOwned()

			local text = nil
			local color = nil
			if on then
				text = lstring( "on" )
				color = pool.get_colorf(0.05, 1.0, 0.0, 1.0)
			else
				text = lstring("off")
				color = pool.get_colorf(1.0, 0.0, 0.0, 1.0)
			end

			local format = {"ninja-text", 35.0, color, 47.0, TextAlign.Center, -1.5, 2.5}
			local fx = text_fx or {drop_shadow = true, shadow_color = colorf( 0, 0, 0, 1.0 ), shadow_offset = v3f( 2, -2, 0), shadow_blur_amount = 100000}

			utility.createText(actor, text, format, fx)
		end
	end

	-- Music volume.
	do
		local on_release =
        function( inner )
            local music_on = not GameOptions.get().music_on

            GameOptions.get().music_on            = music_on
            Audio.get():getPlaylistGroup().volume = music_on and 1.0 or 0.0

            set_volume_texture( inner, music_on )
		end

		local actor = nil
		if engine.default_language_enabled then
			actor  = volume_box.lua:addButton( on_release )
		else
			actor = utility.createButtonWithTexture(volume_box, assets.textures.on_off_button_locale)
			actor.on_release = on_release
		end

		set_volume_texture( actor, GameOptions.get().music_on )
		actor.anchor = make_anchor( Anchor.Anchor_BL, Anchor.Anchor_BL, pool.get_v3f( 0.0, 0.0, 0.0 ) )
	end

	-- Sound volume.
	do
		local on_release =
        function( inner )
            local sound_on = not GameOptions.get().sound_on

            GameOptions.get().sound_on             = sound_on
            Audio.get():getSoundPoolGroup().volume = sound_on and 1.0 or 0.0

            set_volume_texture( inner, sound_on )
        end

		local actor  = nil
		if engine.default_language_enabled then
			actor = volume_box.lua:addButton( on_release )
		else
			actor = utility.createButtonWithTexture(volume_box, assets.textures.on_off_button_locale)
			actor.on_release = on_release
		end

		set_volume_texture( actor, GameOptions.get().sound_on )
		actor.anchor = make_anchor( Anchor.Anchor_BL, Anchor.Anchor_BL, pool.get_v3f( 0.0, 55.0, 0.0 ) )
	end

	local button_box = nil

	-- Box for three middle buttons.
	do
		button_box         = owner:spawn( chunks.window.spawn( Window() ) )
		button_box.size    = pool.get_v2f( 220.0, 130.0 )
		button_box.hidden  = true
		button_box.anchor  = make_anchor( Anchor.Anchor_TL, Anchor.Anchor_C,
		                             pool.get_v3f( self.button_box_offset.x, self.button_box_offset.y, 1.0 ) )
	end

	-- Tutorial.
	do
		local on_release = function( inner )
			local callback = function( o )
				local tutorial = game:getWindow():spawn( chunks.tutorial.spawn() )
				events.on_main_menu_disappeared()
				tutorial.lua:options()
				owner:destroy()
			end

			fade.fade_out( owner, callback )
		end

		local actor = nil
		if engine.default_language_enabled then
			actor  = button_box.lua:addButton( on_release )
			textures.setTexture( actor, assets.textures.tutorial_button )
		else
			local text = lstring( "tutorial" )
			local format = {"ninja-text", 28.0, pool.get_colorf(1.0, 0.0, 0.0, 1.0), 100.0, TextAlign.Center, -1.0, 6.0}
			actor = utility.createButtonWithTextAndTexture(button_box, text, format, assets.textures.options_screen_blank_button )
			actor.on_release = on_release
		end

		actor.anchor = make_anchor( Anchor.Anchor_TL, Anchor.Anchor_TL, pool.get_v3f( 0.0, 0.0, 0.0 ) )
	end

	-- Credits.
	do
		local on_release = function( inner )
			local callback = function( o )
				game:getWindow():spawn( chunks.credits.spawn() )
				events.on_main_menu_disappeared()
				owner:destroy()
			end

			fade.fade_out( owner, callback )
		end

		local actor = nil
		if engine.default_language_enabled then
			actor  = button_box.lua:addButton( on_release )
			textures.setTexture( actor, assets.textures.credits_button )
		else
			local text = lstring( "credits" )
			local format = {"ninja-text", 28.0, pool.get_colorf(1.0, 0.0, 0.0, 1.0), 100.0, TextAlign.Center, -1.0, 6.0}
			actor = utility.createButtonWithTextAndTexture(button_box, text, format, assets.textures.options_screen_blank_button )
			actor.on_release = on_release
		end

		actor.anchor = make_anchor( Anchor.Anchor_TL, Anchor.Anchor_TL, pool.get_v3f( 0.0, -45.0, 0.0 ) )
	end

	-- Restore Purchases
	if not GameService.get():zeb() then
		do
			local on_release = function()
	            if not self.m_iap_handle then
	                self.m_iap_handle = chunks.iap.restorePurchases(function() self.m_iap_handle = nil end)
	            end
			end
	
			local actor = nil
			if engine.default_language_enabled then
				actor  = button_box.lua:addButton( on_release )
				textures.setTexture( actor, assets.textures.restore_button )
			else
				local text_restore = lstring( "restore" )
				local format_restore = {"ninja-text", 28.0, pool.get_colorf(1.0, 0.0, 0.0, 1.0), 100.0, TextAlign.Center, -1.0, 4.0}
	
				if engine.locale_language == "es" or engine.locale_language == "pt" or engine.locale_language == "fr" or engine.locale_language == "it"  or engine.locale_language == "ja" then
	                format_restore[2] = 16.0 -- smaller text for two lines
				    format_restore[7] = 6.0
	
				    if engine.locale_language == "ja" then
				       format_restore[2] = 22.0
				    end
	
				    actor = utility.createButtonWithTextAndTexture(button_box, text_restore, format_restore, assets.textures.options_screen_blank_large_button )
				elseif (engine.locale_language == "ko" or engine.locale_language == "zh-Hans" or engine.locale_language == "zh-Hant") then
				    format_restore[2] = 18.0
				    actor = utility.createButtonWithTextAndTexture(button_box, text_restore, format_restore, assets.textures.options_screen_blank_large_button )
				else
				    format_restore[2] = 18.0
				    actor = utility.createButtonWithTextAndTexture(button_box, text_restore, format_restore, assets.textures.options_screen_blank_button )
				end
				actor.on_release = on_release
			end
	
	
			actor.anchor = make_anchor( Anchor.Anchor_TL, Anchor.Anchor_TL, pool.get_v3f( 0.0, -90.0, 0.0 ) )
		end
	end

	-- Fade in.
	do
		fade.fade_in( owner )
	end
end

function destroy()
    if self.m_iap_handle then
        self.m_iap_handle:cancel()
    end
end

function LuaOptionsMenu:onBack()
    local owner    = self.owner
    local game     = owner:getGame()
    local callback = function( o )
        game:getWindow():spawn( chunks.main.spawn() )
        owner:destroy()
    end

    fade.fade_out( owner, callback )
end

