------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Snake              --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local assets    = chunks.assets
local pool      = chunks.pool
local enemy     = chunks.enemy
local particles = chunks.particles
local actions   = chunks.actions
local effects   = chunks.effects
local utility   = chunks.utility
local textures  = chunks.textures
local sound     = chunks.sound

-- LuaSnake --
LuaSnake = chunks.class.inherit( chunks.enemy.LuaEnemy )
{
	pieces          = nil,
	collision_index = 0
}

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaSnake )

-- The created method.
function LuaSnake:created( owner )
	-- Initialize super.
	enemy.LuaEnemy.created( self, owner )
	
	-- Initialize owner.
	owner.label    = "snake"
	owner.location = pool.get_v3f( -118.0, 0.0, 0.0 )
	
	-- We're not killable.
	self.killable = false
end

-- The collided event.
function LuaSnake:onCollided( other )
	local game   = self.owner:getGame()
	local visual = self.visual
	local split  = SplitAction()
	local delay  = 0.1
	local pop    = function( piece, time )
		local f = function( inner, actor )
			-- Spawn particles.
			do
				particles.spawn_normal( game.lua.root, 
										assets.textures.particles_leaf, 
										actor:getWorldLocation() )
			end
			
			-- Spawn slice.
			do
				effects.spawn_flash( game.lua.root,  
									 assets.textures.effects_slice, 
									 actor:getWorldLocation() )
			end
			
			-- Kill.
			actions.shrink_and_kill( actor )
		end
		
		-- Actions.
		do
			local s = SequenceAction()
			
			s:addAction( SleepAction( time ) )
			s:addAction( LuaAction( 0.0, f, piece ) )
			
			split:addAction( s )
		end
	end
	
	-- Explode initial piece.
	pop( self.pieces[ self.collision_index ], 0.0 )

	-- Explode pieces up.
	if self.collision_index > 1 then
		for i = self.collision_index - 1,1,-1 do
			pop( self.pieces[ i ], math.abs( self.collision_index - i ) * delay )
		end
	end
	
	-- Explode pieces down.
	if self.collision_index < #self.pieces then
		for i = self.collision_index + 1,#self.pieces do
			pop( self.pieces[ i ], math.abs( self.collision_index - i ) * delay )
		end
	end

	-- Add the root action.
	visual:addAction( split )
	
	-- Clear the pieces table.
	self.pieces = {}
	
	-- Play sound.
	sound.play_random( assets.sounds.hit_enemy )
end

-- Returns true if the actors collide.
function LuaSnake:collides( other )
	if self.visual == nil or other.lua.visual == nil then
		return false
	end
	
	local b      = CollidableActor.fast_cast( other.lua.visual )
	local pieces = self.pieces
	
	if b == nil then
		return false
	end
	
	for i = 1,#pieces do
		local a = CollidableActor.fast_cast( pieces[ i ] )
		
		if a ~= nil and a:collides( b ) then
			self.collision_index = i
			
			return true
		end
	end
	
	return false
end

-- Setup snake with the given number of mid segments.
function LuaSnake:setup( num_mids )
	local owner       = self.owner
	local body        = nil
	local head_height = assets.textures.snake_head.size.y
	local mid_height  = assets.textures.snake_mid.size.y
	local tail_height = assets.textures.snake_tail.size.y
	local pieces      = {}
	local index       = 1
	local offset      = 0.0
	
	-- The body (owns segments).
	do
		body = owner:spawn( ActorBase() )
		
		self.visuals.snake = body
		
		self:setVisual( "snake" )
	end

	-- Spawn the head.
	do
		local actor = body:spawn( CollidableActor() )
		
		textures.setTexture( actor, assets.textures.snake_head )
		table.insert( pieces, index, actor )
		
		offset = offset + head_height / 2.0 + mid_height / 2.0
		index  = index + 1
	end
	
	-- Spawn the body pieces.
	do
		for i = 1,num_mids do
			local actor = body:spawn( CollidableActor() )
			
			textures.setTexture( actor, assets.textures.snake_mid )
			actor.location = pool.get_v3f( 0.0, offset, 0.0 )
			table.insert( pieces, index, actor )
			
			-- Only update the offset if it's not the last segment.
			if i < num_mids then
				offset = offset + mid_height
			end
			
			index = index + 1
		end
		
		offset = offset + mid_height / 2.0 + tail_height / 2.0
	end
	
	-- Spawn the tail.
	do
		local actor = body:spawn( CollidableActor() )
		
		textures.setTexture( actor, assets.textures.snake_tail )
		actor.location = pool.get_v3f( 0.0, offset, 0.0 )
		table.insert( pieces, index, actor )
	end
	
	-- Set pieces.
	self.pieces = pieces
end

-- Setup easy sequence.
function LuaSnake:short()
	local owner = self.owner
	
	-- Setup.
	self:setup( 1 )
	
	-- Play sound.
	sound.play_random( assets.sounds.snake_hiss )
	
	-- Actions.
	do
		local s = SequenceAction()
		
		s:addAction( MoveOffsetAction( utility.frandom( 2.0, 2.5 ), InterpAction.Lerp, 
		                               pool.get_v3f( 0.0, -150.0, 0.0 ) ) )
		
		owner:addAction( s )
	end	
end

-- Setup medium sequence.
function LuaSnake:medium()
	local owner = self.owner
	
	-- Setup.
	self:setup( 2 )
	
	-- Play sound.
	sound.play_random( assets.sounds.snake_hiss )
	
	-- Actions.
	do
		local s = SequenceAction()
		
		s:addAction( MoveOffsetAction( utility.frandom( 2.0, 2.5 ) * 1.2, InterpAction.Lerp, 
		                               pool.get_v3f( 0.0, -150.0 * 1.2, 0.0 ) ) )
		
		owner:addAction( s )
	end	
end

-- Setup medium sequence.
function LuaSnake:long()
	local owner = self.owner
	
	-- Setup.
	self:setup( 3 )
	
	-- Play sound.
	sound.play_random( assets.sounds.snake_hiss )
	
	-- Actions.
	do
		local s = SequenceAction()
		
		s:addAction( MoveOffsetAction( utility.frandom( 2.0, 2.5 ) * 1.4, InterpAction.Lerp, 
		                               pool.get_v3f( 0.0, -150.0 * 1.4, 0.0 ) ) )
		
		owner:addAction( s )
	end	
end
