------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Squirrel           --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local assets    = chunks.assets
local pool      = chunks.pool
local enemy     = chunks.enemy
local particles = chunks.particles
local actions   = chunks.actions
local utility   = chunks.utility
local textures  = chunks.textures
local sound     = chunks.sound

-- LuaSquirrel --
LuaSquirrel = chunks.class.inherit( chunks.enemy.LuaEnemy )
{
	offset = -120.0
}

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaSquirrel )

-- The created method.
function LuaSquirrel:created( owner )
	-- Initialize super.
	enemy.LuaEnemy.created( self, owner )
	
	-- Initialize owner.
	owner.label    = "squirrel"
	owner.location = pool.get_v3f( self.offset, 0.0, 0.0 )
	
	-- Thrower.
	do
		local actor = owner:spawn( CollidableActor() )
		
		textures.setTexture( actor, assets.textures.squirrel )

		self.visuals.squirrel = actor
	end
	
	self:setVisual( "squirrel" )
end

-- The collided event.
function LuaSquirrel:onCollided( other, is_bonus )
	local game    = self.owner:getGame()
	local ninja   = game.lua.ninja
	local spawner = is_bonus and particles.spawn_uber or particles.spawn_normal
	
	-- Spawn particles.
	do
		spawner( game.lua.root, 
		         assets.textures.particles_fur, 
		         self.visual:getWorldLocation() )
	end
	
	self.owner:clearActions()
	actions.shrink_and_kill( self.visual )
	
	-- Play sound.
	sound.play_random( assets.sounds.hit_squirrel )
end

-- Setup easy sequence.
function LuaSquirrel:easy()
	local actor = self.owner
	
	-- Owner actions
	do
		local s = SequenceAction()
		local l = LoopAction()
		local f = function( inner, rot )
			self.visual.rotation = pool.get_v3f( 0.0, rot, 0.0 )
		end
		
		s:addAction( SleepAction( utility.frandom( 0.4, 0.6 ) ) )
		l:addAction( LuaAction( 0.0, f, 0.0 ) )
		l:addAction( MoveOffsetAction( utility.frandom( 1.0, 1.4 ), 
		             InterpAction.Lerp, pool.get_v3f( self.offset * -2, 0.0, 0.0 ) ) )         
		l:addAction( SleepAction( utility.frandom( 0.05, 0.15 ) ) )
		l:addAction( LuaAction( 0.0, f, 180.0 ) )
		l:addAction( MoveOffsetAction( utility.frandom( 1.0, 1.4 ),
		             InterpAction.Lerp, pool.get_v3f( self.offset *  2, 0.0, 0.0 ) ) )         
		l:addAction( SleepAction( utility.frandom( 0.05, 0.15 ) ) )
		s:addAction( l )
		
		actor:addAction( s )
	end	
end

-- Setup hard sequence.
function LuaSquirrel:hard()
	local actor = self.owner
	
	-- Actions
	do
		local l = LoopAction()
		local f = function( inner, rot )
			self.visual.rotation = pool.get_v3f( 0.0, rot, 0.0 )
		end
		
		l:addAction( LuaAction( 0.0, f, 0.0 ) )
		l:addAction( MoveOffsetAction( utility.frandom( 1.0, 1.1 ), 
		             InterpAction.Lerp, pool.get_v3f( self.offset * -2, 0.0, 0.0 ) ) )         
		l:addAction( LuaAction( 0.0, f, 180.0 ) )
		l:addAction( MoveOffsetAction( utility.frandom( 0.4, 0.6 ),
		             InterpAction.Lerp, pool.get_v3f( self.offset *  2, 0.0, 0.0 ) ) )         
		l:addAction( SleepAction( utility.frandom( 0.05, 0.15 ) ) )
		
		actor:addAction( l )
	end		
end

-- The bonus setup method.
function LuaSquirrel:bonus( ninja )
	local world       
	local left_offset = ninja.lua.sides.left.offset
	local offset      = v3f( left_offset.x + 10.0, left_offset.y, left_offset.z )
	local ldist       = math.abs( ninja.location.x - offset.x )
	local rdist       = math.abs( ninja.location.x + offset.x )
	local dir         = ldist > rdist and -1 or 1
	local delay       = 0.8
	local sleep_delay = 0.25
	local set_rot     = function( inner, d )
		local r = v3f( 0.0, ( d == 1 ) and 180.0 or 0.0, 0.0 )
		
		ninja.lua:setRotation( r, r )
	end
	
	-- Set visual.
	textures.setTexture( ninja.lua.visual, assets.textures.bonus_squirrel )
	
	-- Enlarge shield.
	ninja.lua.shield.lua:bonusStart( 2.1 )
	
	-- Actions.
	do
		local s     = SequenceAction()
		local split = SplitAction()
		local s1    = SequenceAction()
		local s2    = SequenceAction()
		
		for i = 1,4 do
			-- Rotation.
			s1:addAction( LuaAction( 0.0, set_rot, dir ) )
			
			-- Vertical movement.
			s1:addAction( MoveOffsetAction( delay / 2.0, InterpAction.Smooth, pool.get_v3f( 0.0,  80.0, 0.0 ) ) )
			s1:addAction( MoveOffsetAction( delay / 2.0, InterpAction.Smooth, pool.get_v3f( 0.0, -80.0, 0.0 ) ) )
		
			-- Horizontal movement.
			s2:addAction( MoveAction( delay, InterpAction.Smooth, 
			                          pool.get_v3f( math.abs( offset.x ) * dir, offset.y, offset.z ) ) )
			                          
			-- Sleeps.
			if i < 4 then
				s1:addAction( SleepAction( sleep_delay ) )
				s2:addAction( SleepAction( sleep_delay ) )
			end
			
			dir = dir * -1
		end
		
		split:addAction( s1 )
		split:addAction( s2 )
		s:addAction( split )
		s:addAction( LuaMemberAction( 0.0, "bonusStop" ) )	
		
		ninja:addAction( s )
	end
	
	-- Play sounds.
	sound.play( assets.sounds.bonus_trigger_squirrel )
	sound.play( assets.sounds.bonus_loop_squirrel )
	
	return delay * 4 + sleep_delay * 3
end

-- Update the ninja's trailer.
function LuaSquirrel:updateTrailer( trailer )
	local s = SequenceAction()
			
	s:addAction( ColorAction( 0.4, InterpAction.Smooth, pool.get_colorf( 166.0 / 255.0, 45.0 / 255.0, 0.0, 1.0 ) ) )
			
	trailer:addAction( s )	
end

