------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Bomb               --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local assets    = chunks.assets
local pool      = chunks.pool
local enemy     = chunks.enemy
local particles = chunks.particles
local actions   = chunks.actions
local utility   = chunks.utility
local textures  = chunks.textures
local sound     = chunks.sound

-- LuaBomb --
LuaBomb = chunks.class.inherit( chunks.enemy.LuaEnemy )
{
	exploded   = false,
	fuse_sound = nil
}

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaBomb )

-- The created method.
function LuaBomb:created( owner )
	-- Initialize super.
	enemy.LuaEnemy.created( self, owner )
	
	-- Initialize owner.
	owner.label = "bomb"
	
	-- Thrower.
	do
		local actor = owner:spawn( CollidableActor() )
		self.visuals.bomb = actor
	end
	
	self:setVisual( "bomb" )
end

-- The destroy method.
function LuaBomb:destroy( owner )
	-- Destroy super.
	enemy.LuaEnemy.destroy( self, owner )
	
	-- Stop fuse.
	self:stopFuse()
end

-- Start fuse.
function LuaBomb:startFuse()
	if not self.fuse_sound then
		self.fuse_sound = sound.play( assets.sounds.fuse )
	end
end

-- Stop fuse.
function LuaBomb:stopFuse( owner )
	if self.fuse_sound then
		self.fuse_sound:stop()
		self.fuse_sound = nil
	end
end

-- On game over.
function LuaBomb:onGameOver()
	if self.fuse_sound then
		self.fuse_sound.volume = 0.0
	end
end

-- On pause.
function LuaBomb:onPause()
	if self.fuse_sound then
		self.fuse_sound.volume = 0.0
	end
end

-- On onUnpause.
function LuaBomb:onUnpause()
	if self.fuse_sound then
		self.fuse_sound.volume = 1.0
	end
end

-- The collided event.
function LuaBomb:onCollided( other, is_bonus )
	-- If we've already exploded, do nothing.
	if self.exploded then
		return
	else
		self.exploded = true
	end
	
	local game    = self.owner:getGame()
	local ninja   = game.lua.ninja
	local spawner = is_bonus and particles.spawn_uber or particles.spawn_normal
	
	-- Stop fuse.
	self:stopFuse()
	
	-- Spawn particles.
	do
		spawner( game.lua.root, 
		         assets.textures.particles_bomb, 
		         self.visual:getWorldLocation() )
	end
	
	actions.shrink_and_kill( self.visual )
	
	-- Play sound.
	sound.play_random( assets.sounds.hit_bomb )
end

-- The explode event.
function LuaBomb:explode()
	-- If we've already exploded, do nothing.
	if self.exploded then
		return
	else
		self.exploded = true
	end
	
	local root  = self.owner:getGame().lua.root
	local actor = self.visual
	local world = actor:getWorldLocation()
	
	-- Stop fuse.
	self:stopFuse()
	
	-- Spawn particles.
	do
		particles.spawn_normal( root, 
		                        assets.textures.particles_smoke, 
		                        world )
	end
	
	-- Spawn shrapnel.
	do
		local angles = { 30.0, 150.0, -150.0, -30.0 }
		local delay  = utility.frandom( 1.0, 1.3 )
		local offset = 2.5

		for i = 1,4 do
			local projectile = root:spawn( chunks.star.spawn() )
			local angle      = angles[ i ]
			local target     = utility.random_normal( angle - offset, angle + offset ) * 500.0

			projectile.location = world
			
			-- Actions.
			do
				local s = SequenceAction()
				
				s:addAction( MoveOffsetAction( delay, InterpAction.Lerp, 
				                               pool.get_v3f( target.x, target.y, world.x ) ) )
				s:addAction( ScaleAction( 0.1, InterpAction.Smooth, pool.get_v3f( 0.0, 0.0, 1.0 ) ) )
				s:addAction( DestroyAction( 0.0 ) )
				
				projectile:addAction( s )
			end
		end
	end
	
	-- Need to unregister or ninja collision will fail.
	self:unregister()
	
	-- Shrink and kill.
	actions.shrink_and_kill( actor )

	-- Play sound.
	sound.play_random( assets.sounds.explode )
end

-- The bonus setup method.
function LuaBomb:startBonus( ninja, boost_time, callback )

    local delay       = 0.8
    local cycles = math.floor( boost_time / delay)
    boost_time = cycles * delay
	local left_offset = ninja.lua.sides.left.offset
	local offset      = v3f( -70.0, left_offset.y + 30.0, left_offset.z )
	local ldist       = math.abs( ninja.location.x - offset.x )
	local rdist       = math.abs( ninja.location.x + offset.x )
	local dir         = ldist > rdist and -1 or 1
	
	local set_rot     = function( inner, d )
		local r = ( d == -1 ) and 180.0 or 0.0
		
		ninja.lua:setRotation( v3f( 0.0, r, 0.0 ), v3f( 0.0, r + 180.0, 0.0 ) )
	end
	
	-- Set visual.
	textures.setTexture( ninja.lua.visual, assets.textures.start_boost )
	
	-- Enlarge shield.
	ninja.lua.shield.lua:bonusStart( 3.2 )
	
	-- Actions.
	do
		local s     = SequenceAction()
		local split = SplitAction()
		local s1    = SequenceAction()
		local s2    = SequenceAction()
		
		for i = 1,cycles do
			-- Rotation.
			s1:addAction( LuaAction( 0.0, set_rot, dir ) )
			
			-- Vertical movement.
			s1:addAction( MoveOffsetAction( delay / 2.0, InterpAction.Smooth, pool.get_v3f( 0.0,  50.0, 0.0 ) ) )
			s1:addAction( MoveOffsetAction( delay / 2.0, InterpAction.Smooth, pool.get_v3f( 0.0, -50.0, 0.0 ) ) )
		
			-- Horizontal movement.
			local target = ( i == cycles ) and left_offset or offset
			
			s2:addAction( MoveAction( delay, InterpAction.Smooth, 
			                          pool.get_v3f( math.abs( target.x ) * dir, target.y, target.z ) ) )
			
			dir = dir * -1
		end
		
		split:addAction( s1 )
		split:addAction( s2 )
		s:addAction( split )
		s:addAction( LuaMemberAction( 0.0, "bonusStop" ) )	
		s:addAction( LuaAction( 0.0, callback ) )
		ninja:addAction( s )
	end
	
	-- Turn off the ninja's trailer particles.
	ninja.lua.trailer.particles_per_spawn_range = pool.get_v2i( 0, 0 )
	
	-- Attach a smoke plume to the rocket.
	do
		local particles = ninja:spawn( ParticleSystem() )
		
		-- constant settings
		particles.location                  = pool.get_v3f( 3.0, -5.0, -1.0 )
		particles.shape                     = ParticleSystem.Cone
		particles.texture                   = assets.textures.particles_smoke.path
		particles.particle_size             = assets.textures.particles_smoke.size
		particles.lifetime_range            = pool.get_v2f( 0.4, 0.6 )
		particles.spawn_frequency_range     = pool.get_v2f( 0.04, 0.06 )
		particles.speed_range               = pool.get_v2f( 150.0, 200.0 )
		particles.position_range            = pool.get_v2f( 0.0, 0.0 )
		particles.initial_rotation_range    = pool.get_v2f( 0.0, 360.0 )
		particles.rotation_speed_range      = pool.get_v2f( 80.0, 100.0 )
		particles.color                     = pool.get_colorf( 1.0, 174.0 / 255.0, 0.0, 1.0 )
		particles.follow_system             = true
		particles.cone_angle_range          = pool.get_v2f( -90.0 - 15.0, -90.0 + 15.0 )
		particles.particles_per_spawn_range = pool.get_v2i( 5, 7 )
		particles.explosion                 = true
		particles.lifetime                  = boost_time
		
		-- scale transitions
		particles:addScaleRange( pool.get_v2f( 0.0, 0.0 ) )
		particles:addScaleRange( pool.get_v2f( 0.8, 1.2 ) )
		particles:addScaleRange( pool.get_v2f( 0.8, 1.4 ) )
		particles:addScaleRange( pool.get_v2f( 0.8, 1.2 ) )
		particles:addScaleRange( pool.get_v2f( 0.0, 0.0 ) )
	end
	
	-- Play sounds.
	sound.play( assets.sounds.bonus_trigger_bomb )
	sound.play( assets.sounds.bonus_loop_bomb )

    return boost_time
end

-- Update the ninja's trailer.
function LuaBomb:updateTrailer( trailer )
	local s = SequenceAction()
		
	s:addAction( ColorAction( 0.4, InterpAction.Smooth, 
				 pool.get_colorf( 195.0 / 255.0, 21.0 / 255.0, 22.0 / 255.0, 1.0 ) ) )
			
	trailer:addAction( s )
end