------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Swinger            --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local assets    = chunks.assets
local pool      = chunks.pool
local enemy     = chunks.enemy
local particles = chunks.particles
local actions   = chunks.actions
local utility   = chunks.utility
local textures  = chunks.textures
local sound     = chunks.sound

-- LuaSwinger --
LuaSwinger = chunks.class.inherit( chunks.enemy.LuaEnemy )()

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaSwinger )

-- The created method.
function LuaSwinger:created( owner )
	-- Initialize super.
	enemy.LuaEnemy.created( self, owner )
	
	-- Initialize owner.
	owner.label = "swinger"
	
	-- Thrower.
	do
		local actor = owner:spawn( CollidableActor() )
		
		textures.setTexture( actor, assets.textures.swinger )

		self.visuals.swinger = actor
	end
	
	self:setVisual( "swinger" )
end

-- Bonus meter override.
function LuaSwinger:bonusmeterOverride( actor )
	actor.scale = pool.get_v3f( 0.7, 0.7, 1.0 )

	textures.setTexture( actor, assets.textures.icon_sword )
end	

-- The collided event.
function LuaSwinger:onCollided( other, is_bonus )
	local game    = self.owner:getGame()
	local ninja   = game.lua.ninja
	local spawner = is_bonus and particles.spawn_uber or particles.spawn_normal
	
	-- Spawn particles.
	do
		local system = spawner( self.owner:getGame().lua.root, 
		                        assets.textures.icon_sword,
		                        self.visual:getWorldLocation() )
		                        
		if is_bonus then
			system.lifetime                  = utility.frandom( 0.05, 0.05 ) 
			system.particles_per_spawn_range = pool.get_v2i( 4, 5 )  
			system.spawn_frequency_range     = pool.get_v2f( 0.1, 0.1 )
			
			system.particle_size = pool.get_v2f( system.particle_size.x * 0.8, system.particle_size.y * 0.8 )
		else
			system.lifetime                  = utility.frandom( 0.05, 0.05 ) 
			system.particles_per_spawn_range = pool.get_v2i( 5, 6 )  
			system.spawn_frequency_range     = pool.get_v2f( 0.1, 0.1 )
			
			system.particle_size = pool.get_v2f( system.particle_size.x * 0.5, system.particle_size.y * 0.5 )
		end
		
		
		system:clearScaleRanges()
		system:addColorRange( pool.get_colorf( 1.0, 1.0, 1.0, 1.0 ) )
		system:addColorRange( pool.get_colorf( 1.0, 1.0, 1.0, 1.0 ) )
		system:addColorRange( pool.get_colorf( 1.0, 1.0, 1.0, 0.0 ) )
	end
	
	actions.shrink_and_kill( self.visual )
	
	-- Play sound.
	sound.play_random( assets.sounds.hit_swinger )
end

-- Setup easy sequence.
function LuaSwinger:easy( rope_owner )
	local owner = self.owner
	local delay = utility.frandom( 1.8, 2.2 )
	local f     = function( inner )
		-- Initial location.
		inner.location = pool.get_v3f( -220.0, 150.0, 0.0 )
		
		-- Actions.
		do
			local s     = SequenceAction()
			local split = SplitAction()
			local s1    = SequenceAction()
			local s2    = SequenceAction()
			
			s1:addAction( MoveOffsetAction( delay, InterpAction.Ramp,
											pool.get_v3f( 440.0, 0.0, 0.0 ) ) )
			s2:addAction( MoveOffsetAction( delay, InterpAction.Smooth,
											pool.get_v3f( 0.0, -280.0, 0.0 ) ) )                                
			split:addAction( s1 )
			split:addAction( s2 )
			s:addAction( split )
			
			inner:addAction( s )
		end
	end

	-- Move the swinger.
	f( owner )
	
	-- Spawn the rope.
	do
		local root    = rope_owner:spawn( ActorBase() )
		local subroot = root:spawn( ActorBase() )
		local visual  = subroot:spawn( SpriteActor() )
		
		textures.setTexture( visual, assets.textures.rope )
		
		-- Move the owning actor.
		f( root )
		
		-- Offset the rope.
		subroot.location = pool.get_v3f( 26.3, visual.size.x / 2.0 + 100.0, 0.0 )
		
		-- Adjust positioning.
		visual.scale    = pool.get_v3f( 2.0, 1.0, 1.0 )
		visual.rotation = pool.get_v3f( 0.0, 0.0, 90.0 )
	end
	
	-- Play sound.
	sound.play_random( assets.sounds.swinger_enter )
end

-- Setup hard sequence.
function LuaSwinger:hard( rope_owner )
	local owner = self.owner
	local delay = utility.frandom( 1.2, 1.6 )
	local f     = function( inner )
		-- Initial location.
		inner.location = pool.get_v3f( -220.0, 150.0, 0.0 )
		
		-- Actions.
		do
			local s     = SequenceAction()
			local split = SplitAction()
			local s1    = SequenceAction()
			local s2    = SequenceAction()
			
			s1:addAction( MoveOffsetAction( delay, InterpAction.Ramp,
											pool.get_v3f( 440.0, 0.0, 0.0 ) ) )
			s2:addAction( MoveOffsetAction( delay, InterpAction.Smooth,
											pool.get_v3f( 0.0, -280.0, 0.0 ) ) )                                
			split:addAction( s1 )
			split:addAction( s2 )
			s:addAction( split )
			
			inner:addAction( s )
		end
	end

	-- Move the swinger.
	f( owner )
	
	-- Spawn the rope.
	do
		local root    = rope_owner:spawn( ActorBase() )
		local subroot = root:spawn( ActorBase() )
		local visual  = subroot:spawn( SpriteActor() )
		
		textures.setTexture( visual, assets.textures.rope )
		
		-- Move the owning actor.
		f( root )
		
		-- Offset the rope.
		subroot.location = pool.get_v3f( 26.3, visual.size.x / 2.0 + 100.0, 0.0 )
		
		-- Adjust positioning.
		visual.scale    = pool.get_v3f( 2.0, 1.0, 1.0 )
		visual.rotation = pool.get_v3f( 0.0, 0.0, 90.0 )
	end
	
	-- Play sound.
	sound.play_random( assets.sounds.swinger_enter )
end

-- The bonus setup method.
function LuaSwinger:bonus( ninja )
	local offset  = ninja.lua.sides.left.offset
	local dir     = ( ninja:getWorldLocation().x < 0.0 ) and 1 or -1
	local delay   = 0.5
	local pivot   = v3f( 50.0, -100.0, offset.z )
	local target  = v3f( offset.x, offset.y + 70.0, offset.z )
	local set_rot = function( inner, d )
		local r = v3f( 0.0, ( d == 1 ) and 180.0 or 0.0, 0.0 )
		
		ninja.lua:setRotation( v3f( 0.0, 0.0, 0.0 ), r )
	end
	
	textures.setTexture( ninja.lua.visual, assets.textures.bonus_swinger )
	
	-- Enlarge shield.
	ninja.lua.shield.lua:bonusStart( 2.3 )
	
	-- Actions.
	do
		local s     = SequenceAction()
		local split = SplitAction()
		local s1    = SequenceAction()
		local s2    = SequenceAction()
				
		-- Vertical actions.
		s1:addAction( MoveOffsetAction( delay * 3.5, InterpAction.Smooth,
		                                pool.get_v3f( 0.0, 300.0, 0.0 ) ) )
		s1:addAction( MoveOffsetAction( delay * 3.5, InterpAction.Smooth,
		                                pool.get_v3f( 0.0, -300.0, 0.0 ) ) )
		                                
		-- Trajectory.
		for i = 1,7 do
			-- Update rotation.
			s2:addAction( LuaAction( 0.0, set_rot, dir ) )
			
			-- If first, move to correct offset.
			if i == 1 then
				s2:addAction( MoveAction( delay, InterpAction.Smooth,
		                                  pool.get_v3f( math.abs( pivot.x ) * dir, pivot.y, pivot.z ) ) )
			-- If last, move to end offset.
			elseif i == 7 then
				s2:addAction( MoveAction( delay, InterpAction.Smooth,
		                                  pool.get_v3f( math.abs( target.x ) * dir, target.y, target.z ) ) )
			-- Otherwise, just move back and forth.
			else
				s2:addAction( MoveOffsetAction( delay, InterpAction.Smooth,
		                                        pool.get_v3f( math.abs( pivot.x ) * 2.0 * dir, 0.0, 0.0 ) ) )
			end
			
			dir = dir * -1
		end
		
		dir = dir * -1
		
		split:addAction( s1 )
		split:addAction( s2 )
		s:addAction( split )
		s:addAction( LuaMemberAction( 0.0, "bonusStop" ) )
		
		ninja:addAction( s )
	end
	
	-- Play sounds.
	sound.play( assets.sounds.bonus_trigger_swinger )
	sound.play( assets.sounds.bonus_loop_swinger )

	return delay * 7
end

-- Update the ninja's trailer.
function LuaSwinger:updateTrailer( trailer )
	local s = SequenceAction()
			
	s:addAction( ColorAction( 0.4, InterpAction.Smooth, pool.get_colorf( 1.0, 1.0, 1.0, 1.0 ) ) )
			
	trailer:addAction( s )	
end