------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Thrower            --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local assets    = chunks.assets
local pool      = chunks.pool
local enemy     = chunks.enemy
local particles = chunks.particles
local actions   = chunks.actions
local effects   = chunks.effects
local utility   = chunks.utility
local textures  = chunks.textures
local sound     = chunks.sound
local constants = engine.platform.constants

-- LuaThrower --
LuaThrower = chunks.class.inherit( chunks.enemy.LuaEnemy )()

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaThrower )

-- The created method.
function LuaThrower:created( owner )
	-- Initialize super.
	enemy.LuaEnemy.created( self, owner )
	
	-- Initialize owner.
	owner.label    = "thrower"
	owner.location = pool.get_v3f( -120.0, 0.0, 0.0 )
	
	-- Thrower.
	do
		local actor = owner:spawn( CollidableActor() )
		
		textures.setTexture( actor, assets.textures.thrower )

		self.visuals.thrower = actor
	end
	
	self:setVisual( "thrower" )
	
	-- We're not killable.
	self.killable = false
end

-- The collided event.
function LuaThrower:onCollided( other )
	-- Spawn particles.
	do
		particles.spawn_normal( self.owner:getGame().lua.root, 
		                        assets.textures.particles_spark, 
		                        self.visual:getWorldLocation() )
	end
	
	-- Spawn slice.
	do
		effects.spawn_flash( self.owner:getGame().lua.root, 
		                     assets.textures.effects_slice, 
		                     self.visual:getWorldLocation() )
	end
	
	actions.shrink_and_kill( self.visual )
	
	-- Play sound.
	sound.play_random( assets.sounds.hit_enemy )
end

-- Setup single thrower.
function LuaThrower:single()
	local actor = self.owner
	
	-- Actions
	do
		local s = SequenceAction()
		local f = function( inner )
			local projectile = actor:spawn( chunks.star.spawn() )
			local target     = utility.random_normal( 0.0, 10.0 ) * 350.0
			
			-- Actions
			do
				local s2 = SequenceAction()
				
				s2:addAction( MoveOffsetAction( utility.frandom( 1.8, 2.0 ), 
				              InterpAction.Lerp, pool.get_v3f( target.x, target.y, 0.0 ) ) )
				
				projectile:addAction( s2 )
			end
			
			-- Play sound.
			sound.play_random( assets.sounds.throw )
		end
		
		s:addAction( SleepAction( 0.5 ) )
		s:addAction( LuaAction( 0.0, f ) )
		
		actor:addAction( s )
	end	
end

-- Setup double thrower.
function LuaThrower:double()
	local actor = self.owner
	
	-- Actions
	do
		local s = SequenceAction()
		local f = function( inner )
			local lifetime = utility.frandom( 1.4, 1.6 )
			local targets  = { utility.random_normal(  10.0,  15.0 ) * 350.0,
			                   utility.random_normal( -10.0, -15.0 ) * 350.0 }
			                  
			for i in ipairs( targets ) do
				local projectile = actor:spawn( chunks.star.spawn() )
				local target     = targets[ i ]
				
				-- Actions
				do
					local s2 = SequenceAction()
					
					s2:addAction( MoveOffsetAction( lifetime, 
								  InterpAction.Lerp, pool.get_v3f( target.x, target.y, 0.0 ) ) )
					
					projectile:addAction( s2 )
				end
				
				-- Play sound.
				sound.play_random( assets.sounds.throw )
			end
		end
		
		s:addAction( SleepAction( 0.6 ) )
		s:addAction( LuaAction( 0.0, f ) )
		
		actor:addAction( s )
	end	
end

-- Setup bomb thrower.
function LuaThrower:bomb()
	local actor = self.owner
	
	-- Actions
	do
		local s = SequenceAction()
		local f = function( inner )
			local projectile = actor:spawn( chunks.bomb.spawn() )
			
			-- Actions
			do
				local s2      = SequenceAction()
				local split   = SplitAction()
				local s3      = SequenceAction()
				local delay   = utility.frandom( 1.0, 1.2 )
				local target  = utility.random_normal( 10.0, 20.0 ) * utility.frandom( 175.0, 185.0 )
				local explode = function( inner )
					inner.lua:explode()
				end

				s2:addAction( MoveOffsetAction( delay, InterpAction.Lerp, pool.get_v3f( target.x, target.y, 0.0 ) ) )
				s2:addAction( LuaAction( 0.0, explode ) )
				
				projectile:addAction( s2 )
			end
			
			-- Play sound.
			sound.play_random( assets.sounds.throw )
		end
		
		s:addAction( SleepAction( 0.4 ) )
		if constants.WIDESCREEN then
			s:addAction( SleepAction( 0.2 ) )
		end
		s:addAction( LuaAction( 0.0, f ) )
		
		actor:addAction( s )
	end
end
