------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Troll              --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local assets    = chunks.assets
local pool      = chunks.pool
local enemy     = chunks.enemy
local particles = chunks.particles
local actions   = chunks.actions
local utility   = chunks.utility
local textures  = chunks.textures
local sound     = chunks.sound
local effects   = chunks.effects

-- LuaTroll --
LuaTroll = chunks.class.inherit( chunks.enemy.LuaEnemy )
{
	offset = -120.0
}

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaTroll )

-- The created method.
function LuaTroll:created( owner )
	-- Initialize super.
	enemy.LuaEnemy.created( self, owner )
	
	-- Initialize owner.
	owner.label    = "troll"
	owner.location = pool.get_v3f( self.offset, 0.0, 0.0 )
	
	-- Thrower.
	do
		local actor = owner:spawn( CollidableActor() )
		
		textures.setTexture( actor, assets.textures.troll )

		self.visuals.troll = actor
	end
	
	self:setVisual( "troll" )
end

-- Bonus meter override.
function LuaTroll:bonusmeterOverride( actor )
    actor.scale = pool.get_v3f( 0.9, 0.9, 1.0 )
    
    textures.setTexture( actor, assets.textures.troll )
end

-- The collided event.
function LuaTroll:onCollided( other, is_bonus )
	local game    = self.owner:getGame()
	local ninja   = game.lua.ninja
	local spawner = is_bonus and particles.spawn_big or particles.spawn_normal
	
	-- Spawn slice.
	do
		effects.spawn_flash( self.owner:getGame().lua.root, 
		                     assets.textures.effects_slice, 
		                     self.visual:getWorldLocation() )
	end
	
	-- Spawn particles.
	do
		local system = spawner( self.owner:getGame().lua.root, 
		                        assets.textures.particles_club,
		                        self.visual:getWorldLocation() )
		
		system:clearScaleRanges()
		system:addColorRange( pool.get_colorf( 1.0, 1.0, 1.0, 1.0 ) )
		system:addColorRange( pool.get_colorf( 1.0, 1.0, 1.0, 1.0 ) )
		system:addColorRange( pool.get_colorf( 1.0, 1.0, 1.0, 0.0 ) )
	end
	
	self.owner:clearActions()
	actions.shrink_and_kill( self.visual )
	
	-- Play sound.
	sound.play_random( assets.sounds.hit_troll )
end

-- Setup easy sequence.
function LuaTroll:easy()
	local actor = self.owner
	
	-- Owner actions
	do
		local s      = SequenceAction()
		local l      = LoopAction()
		local delay1 = utility.frandom( 1.0, 1.6 )
		local delay2 = utility.frandom( 1.0, 1.6 )
		local f      = function( inner, rot )
			self.visual.rotation = pool.get_v3f( 0.0, rot, 0.0 )
		end
		
		-- Initial sleep, setup rotation.
		s:addAction( SleepAction( utility.frandom( 0.5, 0.6 ) ) )
		l:addAction( LuaAction( 0.0, f, 0.0 ) )
		
		-- Move across.
		l:addAction( MoveOffsetAction( delay1, InterpAction.Lerp, pool.get_v3f( self.offset * -2, 0.0, 0.0 ) ) )    
		
		-- Sleep for a second.
		l:addAction( SleepAction( utility.frandom( 0.05, 0.15 ) ) )
		l:addAction( LuaAction( 0.0, f, 180.0 ) )
		
		-- Move back.
		l:addAction( MoveOffsetAction( delay2, InterpAction.Lerp, pool.get_v3f( self.offset * 2, 0.0, 0.0 ) ) )
		
		-- Sleep for a second.
		l:addAction( SleepAction( utility.frandom( 0.05, 0.15 ) ) )
		s:addAction( l )
		
		actor:addAction( s )
	end	
	
	-- Play growl sound.
	sound.play_random( assets.sounds.troll_growl )
end

-- Setup easy sequence.
function LuaTroll:hard()
	local actor = self.owner
	
	-- Owner actions
	do
		local s      = SequenceAction()
		local l      = LoopAction()
		local delay1 = utility.frandom( 0.85, 0.85 )
		local delay2 = utility.frandom( 0.4, 0.6 )
		local f      = function( inner, rot )
			self.visual.rotation = pool.get_v3f( 0.0, rot, 0.0 )
		end
		
		-- Initial sleep, setup rotation.
		s:addAction( SleepAction( utility.frandom( 0.3, 0.3 ) ) )
		l:addAction( LuaAction( 0.0, f, 0.0 ) )
		
		-- Move across.
		l:addAction( MoveOffsetAction( delay1, InterpAction.Lerp, pool.get_v3f( self.offset * -2, 0.0, 0.0 ) ) )    
		
		-- Sleep for a second.
		l:addAction( SleepAction( utility.frandom( 0.05, 0.15 ) ) )
		l:addAction( LuaAction( 0.0, f, 180.0 ) )
		
		-- Move back.
		l:addAction( MoveOffsetAction( delay2, InterpAction.Lerp, pool.get_v3f( self.offset * 2, 0.0, 0.0 ) ) )
		
		-- Sleep for a second.
		l:addAction( SleepAction( utility.frandom( 0.05, 0.15 ) ) )
		s:addAction( l )
		
		actor:addAction( s )
	end	
	
	-- Play growl sound.
	sound.play_random( assets.sounds.troll_growl )
end

-- The bonus setup method.
function LuaTroll:bonus( ninja )
	local game        = ninja:getGame()
	local offset      = ninja.lua.sides.left.offset
	local dir         = ( ninja:getWorldLocation().x < 0.0 ) and -1 or 1
	local frame_time  = 0.0769231
	local up_delay    = frame_time * 6
	local down_delay  = frame_time * 4
	local vdist       = 160.0
	local hoffset     = 93.5
	local set_rot     = function( inner, d )
		local r = v3f( 0.0, ( d == 1 ) and 180.0 or 0.0, 0.0 )
		
		ninja.lua:setRotation( r, r )
	end
	local jump = function( inner )
		sound.play_random( assets.sounds.ninja_jump )
		
		-- Spawn particles.
		do
			local system = ninja.lua.visual:spawn( ParticleSystem() )
	
			-- Set explosion parameters.
			system.texture                   = assets.textures.bonus_troll.path
			system.particle_size             = assets.textures.bonus_troll.size
			system.explosion                 = true
			system.lifetime                  = up_delay
			system.lifetime_range            = pool.get_v2f( 0.4, 0.4 )
			system.particles_per_spawn_range = pool.get_v2i( 1, 1 )
			system.spawn_frequency_range     = pool.get_v2f( 0.4 * 0.04, 0.4 * 0.04 )
				
			-- color
			system:addColorRange( pool.get_colorf( 1.0, 1.0, 1.0, 1.0 ) )
			system:addColorRange( pool.get_colorf( 1.0, 1.0, 1.0, 0.0 ) )	
		end
	end
	
	local rotation = v3f( 0.0, ( dir == -1 ) and 0.0 or 180.0, 0.0 )
	
	-- Set visual.
	textures.setTexture( ninja.lua.visual, assets.textures.bonus_troll )
	
	-- Enlarge shield.
	ninja.lua.shield.lua:bonusStart( 2.5 )
	
	-- Set location.
	ninja.location = pool.get_v3f( hoffset * dir, offset.y, offset.z )
	
	dir = dir * -1
	
	-- Actions.
	do
		local s = SequenceAction()
		
		for i = 1,4 do
			local split = SplitAction()
			
			s:addAction( LuaAction( 0.0, set_rot, dir ) )
			s:addAction( LuaAction( 0.0, jump ) )
			split:addAction( MoveOffsetAction( up_delay, InterpAction.Smooth,
			                                   pool.get_v3f( 0.0, vdist, 0.0 ) ) )
			split:addAction( MoveOffsetAction( up_delay, InterpAction.Smooth,
			                                   pool.get_v3f( hoffset * dir * 2.0, 0.0, 0.0 ) ) )
			s:addAction( split )
			s:addAction( MoveOffsetAction( down_delay, InterpAction.Smooth,
			                               pool.get_v3f( 0.0, -vdist, 0.0 ) ) )
			                               
			dir = dir * -1                               
		end

		s:addAction( LuaMemberAction( 0.0, "bonusStop" ) )
		
		ninja:addAction( s )
	end
	
	-- Play sound.
	sound.play( assets.sounds.bonus_troll )
	
	return up_delay * 4 + down_delay * 4
end

-- Update the ninja's trailer.
function LuaTroll:updateTrailer( trailer )
	local s = SequenceAction()
			
	s:addAction( ColorAction( 0.4, InterpAction.Smooth, 
	             pool.get_colorf( 80.0 / 255.0, 245.0 / 255.0, 34.0 / 255.0, 1.0 ) ) )
	
	trailer:addAction( s )	
end

